﻿using System;
using System.Net;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET.InternalArgs;

namespace GMAS_Lib_test
{
    
    public partial class ConnectivityControl : UserControl, IParentPanel
    {
        public ConnectivityControl()
        {
            InitializeComponent();
        }

        private Panel panel;
        public Panel GetPanel()
        {
            return panel;
        }

        public void Init( ConnectivityData data, Panel pnl )
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
            ConnectData_OnDataChange();
        }

        void ConnectData_OnDataChange()
        {
            EnableButtons(ConnectData.Handle != 0 );
           
        }
        private void EnableButtons( bool enable )
        {
            btnAxisConnect.Enabled=enable;
            btnGroupConnect.Enabled = enable;
            btnGetVersion.Enabled = enable;
            btnNetwork.Enabled = enable;
            chbWarningsEnable.Enabled = enable;
        }
        private void OnVisibleChange(object sender, EventArgs e)
        {

        }

        private void OnLoad(object sender, EventArgs e)
        {
            Visible = true;
            Dock = DockStyle.Fill;
        }

        public static string TranslateAsyncEvent(eASYNC_EVENT val)
        {
            switch (val)
            {
                case eASYNC_EVENT.eASYNC_EVENT_SDO_DOWNLOAD:
                    return "SDO DOWNLOAD";
                case eASYNC_EVENT.eASYNC_EVENT_SDO_UPLOAD:
                    return "SDO UPLOAD";
                case eASYNC_EVENT.eASYNC_EVENT_CHANGE_MOTION_MODE:
                    return "CHANGE MOTION MODE";
                case eASYNC_EVENT.eASYNC_EVENT_INTERP_CMD_GET:
                    return "INTERP CMD GET";
                case eASYNC_EVENT.eASYNC_EVENT_READ_DI_GROUP:
                    return "READ DI GROUP";
                case eASYNC_EVENT.eASYNC_EVENT_WRITE_DO_GROUP:
                    return "WRITE DO GROUP";
                case eASYNC_EVENT.eASYNC_EVENT_CONFIG_REG_PARAM_EVENT_PDO3:
                    return "CONFIG REG PARAM EVENT PDO3";
                case eASYNC_EVENT.eASYNC_EVENT_CONFIG_REG_PARAM_EVENT_PDO4:
                    return "CONFIG REG PARAM EVENT PDO4";
                case eASYNC_EVENT.eASYNC_EVENT_CONFIG_USER_PARAM_EVENT_PDO3:
                    return "CONFIG USER PARAM EVENT PDO3";
                case eASYNC_EVENT.eASYNC_EVENT_CONFIG_USER_PARAM_EVENT_PDO4:
                    return "CONFIG USER PARAM EVENT PDO4";
                case eASYNC_EVENT.eASYNC_EVENT_CANCEL_PARAM_EVENT_PDO3:
                    return "CANCEL PARAM EVENT PDO3";
                case eASYNC_EVENT.eASYNC_EVENT_CANCEL_PARAM_EVENT_PDO4:
                    return "CANCEL PARAM EVENT PDO4";
                case eASYNC_EVENT.eASYNC_EVENT_RESET:
                    return "RESET";
                case eASYNC_EVENT.eASYNC_EVENT_INTERP_CMD_SET:
                    return "INTERP CMD SET";
                case eASYNC_EVENT.eASYNC_EVENT_SEND_RAW_DATA:
                    return "SEND RAW DATA";
                case eASYNC_EVENT.eASYNC_EVENT_EXECUTE_LABEL:
                    return "EXECUTE LABEL";
                case eASYNC_EVENT.eASYNC_EVENT_CONFIG_PDO_COMM_PARAM:
                    return "CONFIG PDO COMM PARAM";
                case eASYNC_EVENT.eASYNC_EVENT_CONFIG_VIRTUAL_ENC:
                    return "CONFIG VIRTUAL ENC";
                case eASYNC_EVENT.eASYNC_EVENT_BULK_UPLOAD:
                    return "BULK UPLOAD";
                case eASYNC_EVENT.eASYNC_EVENT_SDO_DOWNLOAD_MOTION_PROCESS:
                    return "SDO DOWNLOAD MOTION PROCESS";
                case eASYNC_EVENT.eASYNC_EVENT_SDO_UPLOAD_MOTION_PROCESS:
                    return "SDO UPLOAD MOTION PROCESS";
                default:
                    return "UNDEFINED";
            }
        }

        private void UserCallback(object sender, MMC_CAN_REPLY_DATA_OUT data)
        {
            eASYNC_EVENT evType = (eASYNC_EVENT) data.btEventType;
            if (evType != eASYNC_EVENT.eASYNC_EVENT_UNDEFINED)
            {
                string evName = TranslateAsyncEvent(evType);
                if (InvokeRequired)
                {
                    Invoke(new ToOutputDelegate(ToOutput), new object[1] { evName });
                }
                else
                {
                    ConnectData.ToOutput(evName);
                }
            }
            MMCConnection cnt = sender as MMCConnection;
            bool needSwap = cnt != null && cnt.IsBigEndian;
            ushort axRef = needSwap ? SwappedParam.Swap(data.usAxisRef) : data.usAxisRef;
            ushort cobID = needSwap ? SwappedParam.Swap(data.usCOB_ID) : data.usCOB_ID;
            short errID = needSwap ? SwappedParam.Swap(data.usErrorID) : data.usErrorID;
            ushort status = needSwap ? SwappedParam.Swap(data.usStatus) : data.usStatus;
            if (axRef == ConnectData.SingleAxis.AxisReference)
            {
                ConnectData.AsyncResponse++;
                MC_FUNC_IDS fID = (MC_FUNC_IDS) (needSwap ? SwappedParam.Swap(data.usFunctionID) : data.usFunctionID);
                switch( fID )
                {
                    case MC_FUNC_IDS.ASYNC_REPLY_EVT:
                        if (errID != 0)
                        {
                            MMCException.Throw( ConnectData.Handle, "ElmoGetSyncFloatParam DriveError",
                                               (short)IDCommands.MC_INTERPCMD_ID,
                                               MMCConnection.GetConnection(ConnectData.Handle).DestinationAddress,
                                               ConnectData.SingleAxis.AxisName, ConnectData.SingleAxis.AxisReference,
                                               LibraryErrors.NoError, (MMCErrors)(errID), status, data.data);
                        }
                         bool float_type = false;
                         String str = "Async callback accepted from Binary Interpreter. " +
                                     MMCConnection.GetElmoCommandName(data) +
                                         MMCConnection.GetElmoCommandIndex(data) + "=";
                        Int16 shIndex = BitConverter.ToInt16(data.data, 2);
                        int iVal = BitConverter.ToInt32(data.data, 4);
                        if ((shIndex & 0x8000) != 0)
                            float_type = true;
                    
                        if( !float_type )
                            str += iVal.ToString();
                        else
                        {
                            float fVal = BitConverter.ToSingle(data.data, 4);
                            str += fVal.ToString();
                        }
                        if(InvokeRequired)
                        {
                            object[] objArr = new object[1];
                            objArr[0] = str;
                            Invoke(new ToOutputDelegate(ToOutput), objArr);
                        }
                        else
                            ConnectData.ToOutput(str);
                    break;
                    case MC_FUNC_IDS.DOWNLOAD_FW_EVT:
                        String strDF = string.Format("Download Firmware event accepted from axis {0}(AxisRef={1})",
                            ConnectData.SingleAxis.AxisName, ConnectData.SingleAxis.AxisReference);
                        if (InvokeRequired)
                        {
                            object[] objArr = new object[1];
                            objArr[0] = strDF;
                            Invoke(new ToOutputDelegate(ToOutput), objArr);
                        }
                        else
                            ConnectData.ToOutput(strDF);
                        break;
                    
                    case MC_FUNC_IDS.DRVERROR_EVT:
                        String strDRVERROR = string.Format("Drive Error event accepted from axis {0}(AxisRef={1})",
                            ConnectData.SingleAxis.AxisName, ConnectData.SingleAxis.AxisReference);
                        if (InvokeRequired)
                        {
                            object[] objArr = new object[1];
                            objArr[0] = strDRVERROR;
                            Invoke(new ToOutputDelegate(ToOutput), objArr);
                        }
                        else
                            ConnectData.ToOutput(strDRVERROR);
                        break;
                    case MC_FUNC_IDS.EMIT_EVT:
                        String strEMIT = string.Format("Emit event accepted from axis {0}(AxisRef={1})", 
                            ConnectData.SingleAxis.AxisName, ConnectData.SingleAxis.AxisReference);
                        if (InvokeRequired)
                        {
                            object[] objArr = new object[1];
                            objArr[0] = strEMIT;
                            Invoke(new ToOutputDelegate(ToOutput), objArr);
                        }
                        else
                            ConnectData.ToOutput(strEMIT);
                        break;
                    
                    case MC_FUNC_IDS.SYSTEMERROR_EVT:
                        String strSYSTEMERR = string.Format("System Error event accepted from axis {0}(AxisRef={1})", 
                            ConnectData.SingleAxis.AxisName, ConnectData.SingleAxis.AxisReference);
                        if (InvokeRequired)
                        {
                            object[] objArr = new object[1];
                            objArr[0] = strSYSTEMERR;
                            Invoke(new ToOutputDelegate(ToOutput), objArr);
                        }
                        else
                            ConnectData.ToOutput(strSYSTEMERR);
                        break;
                }
               
            }
        }

        private delegate void ToOutputDelegate(Object obj);
        private void ToOutput( Object obj )
        {
            ConnectData.ToOutput( obj);
        }
        private void ConnectClick(object sender, EventArgs e)
        {
            Int32 hndl;
            IPAddress rIP;
            if (!IPAddress.TryParse(tb_RIP.Text, out rIP))
            {
                MessageBox.Show("IP Format Error:Wrong Remote IP", "IP Format Error");
                return;
            }

            IPAddress lIP;
            if (!IPAddress.TryParse(tb_LIP.Text, out lIP))
            {
                MessageBox.Show("IP Format Error:Wrong Local IP", "IP Format Error");
                return;
            }
            if (MMCConnection.ConnectRPC(rIP, 4000, lIP, 5000, new cbFunc(UserCallback), 0xEFFFFFFF, out hndl) != 0)
            {
                MessageBox.Show("Connection failed. Check network configuration.", "Connection Failed");
                return;
            }
            ConnectData.Handle = hndl;
            MMCConnection.RegisterEndMotionEventCallback(hndl, new MotionEndEvent(OnEndMotionEvent));
            MMCConnection.RegisterEndHomingEventCallback(hndl, new HomingEndEvent(OnEndHomingEvent));
            MMCConnection.RegisterHBeatEventCallback(hndl, new HBeatEventCallback(OnHBeatEvent));
            MMCConnection.RegisterPdoRcvEventCallback(hndl, new PdoRcvEventCallback(OnPDORcv));
            MMCConnection.RegisterEmergencyEventCallback(hndl, new EmergencyEventCallback(OnEmergencyRcv));
            MMCConnection.RegisterTouchProbeEndCallback(hndl, new TouchProbeEndCallback(OnTouchProbeEnd));
            MMCConnection.RegisterModbusWriteCallback(hndl, new ModbusWriteCallback(OnModbusWrite));
            MMCConnection.RegisterErrorStateCallback(hndl, new ErrorStateEventCallback(OnErrorStateEvent));
            MMCConnection.RegisterUserProgramCallback(hndl, new UserProgramEventCallback(OnUserProgramEvent));
            MMCConnection.RegisterUDPDataErrorCallback(hndl, new UDPCallbackDataError(OnUDPCallbackDataError));
            MMCConnection.EnableWarning(hndl, chbWarningsEnable.Checked);
            MMCConnection.GetUDPListenerPortNumber(hndl);
            //ConnectData.IsValid = true;

            btnAxisConnect.Enabled = true;
            btnGroupConnect.Enabled = true;
            btnDisconnect.Enabled = true;

            ConnectData.IsCANInternal = MMCConnection.ReadGlobalBoolParameter(hndl, 
                MMC_BOOLEAN_PARAMETER_LIST_ENUM.MMC_CONNECTION_TYPE_PARAM, 0, 1) == 2;//CAN - 2, EtherCAT - 1
            if (ConnectData.IsCANInternal) 
            {
                rdbCAN.Checked = true;
                rdbEth.Checked = false;
            }
            else
            {
                rdbCAN.Checked = false;
                rdbEth.Checked = true;
            }
            
            ConnectData.ToOutput("Connection successfully established.");
        }

        private void OnEndMotionEvent(UInt16 axRef, bool res )
        {
            String strME = "End Motion event accepted from axis = " + axRef.ToString() +
                                      " Result of Motion = " + res.ToString();
            if (InvokeRequired)
            {
                object[] objArr = new object[1];
                objArr[0] = strME;
                Invoke(new ToOutputDelegate(ToOutput), objArr);
            }
            else
                ConnectData.ToOutput(strME);
        }

        private void OnErrorStateEvent(UInt16 axRef, Int16 state, UInt16 emgCode)
        {
            String strME = "Error State event accepted from axis = " + axRef.ToString() + " State mask = " +
                           state.ToString("X") + " Emergency Code = " + emgCode.ToString("X");
                                      
            if (InvokeRequired)
            {
                object[] objArr = new object[1];
                objArr[0] = strME;
                Invoke(new ToOutputDelegate(ToOutput), objArr);
            }
            else
                ConnectData.ToOutput(strME);
        }

        private void OnUserProgramEvent( byte[] data )
        {
            if( data != null )
            {
                UserEventData eventData = new UserEventData();
                try
                {
                    eventData.Init( data);
                }
                catch (Exception)
                {
                    
                    
                }
            }
        }
        private void OnUDPCallbackDataError( MC_FUNC_IDS fID, byte[] data)
        {
            String str;
            if( fID == MC_FUNC_IDS.UNKNOWN )
            {
                str = "Unknown Function ID on Callback received\nAccepted Data=";
            }
            else
            {
                str = "Invalid data received on Callback with FID=" + fID.ToString() + "\nAccepted Data=";
            }
            char[] cArr = new char[data.Length];
            for (int i = 0; i < cArr.Length; i++)
                cArr[i] = (char)data[i];
            str += new string( cArr);
                if (InvokeRequired)
                {
                    object[] objArr = new object[1];
                    objArr[0] = str;
                    Invoke(new ToOutputDelegate(ToOutput), objArr);
                }
                else
                    ConnectData.ToOutput(str);
            

        }
        private void OnEndHomingEvent(UInt16 axRef, Int16 err)
        {
            String str = "End Homing event accepted from axis = " + axRef.ToString() +
                                      " Homing Error = " + err.ToString();
            if (InvokeRequired)
            {
                object[] objArr = new object[1];
                objArr[0] = str;
                Invoke(new ToOutputDelegate(ToOutput), objArr);
            }
            else
                ConnectData.ToOutput(str);
        }
        private void OnHBeatEvent(UInt16 axRef)
        {
            String str = "Heartbeat event accepted from axis = " + axRef.ToString();
            if (InvokeRequired)
            {
                object[] objArr = new object[1];
                objArr[0] = str;
                Invoke(new ToOutputDelegate(ToOutput), objArr);
            }
            else
                ConnectData.ToOutput(str);
        }

        private void OnPDORcv( UInt16 axRef, UInt16 EvGroup, UNPDODAT data1, UNPDODAT data2 )
        {
            String str = "PDO event accepted from axis = " + axRef.ToString();
            if (InvokeRequired)
            {
                object[] objArr = new object[1];
                objArr[0] = str;
                Invoke(new ToOutputDelegate(ToOutput), objArr);
            }
            else
                ConnectData.ToOutput(str);
        }

        private void OnEmergencyRcv(UInt16 axRef, UInt16 emgCode)
        {
            String str = "Emergency accepted from axis = " + axRef.ToString() +
                                      " Emergency Code = " + emgCode.ToString("X");
            if (InvokeRequired)
            {
                object[] objArr = new object[1];
                objArr[0] = str;
                Invoke(new ToOutputDelegate(ToOutput), objArr);
            }
            else
                ConnectData.ToOutput(str);
        }
        
        private void OnTouchProbeEnd(UInt16 axRef, long position)
        {
            String str = "Touch Probe End event accepted from axis = " + axRef.ToString() +
                                      " In position = " + position.ToString();
            if (InvokeRequired)
            {
                object[] objArr = new object[1];
                objArr[0] = str;
                Invoke(new ToOutputDelegate(ToOutput), objArr);
            }
            else
                ConnectData.ToOutput(str);
        }

        private void OnModbusWrite()
        {
            String str = "Modbus Write event accepted";
            if (InvokeRequired)
            {
                object[] objArr = new object[1];
                objArr[0] = str;
                Invoke(new ToOutputDelegate(ToOutput), objArr);
            }
            else
                ConnectData.ToOutput(str);
        }
        private void DisconnectClick(object sender, EventArgs e)
        {
            if (ConnectData == null || ConnectData.Handle == 0)
                return;
            try
            {
                MMCConnection.CloseConnection(MMCConnection.GetConnection(ConnectData.Handle));
                ConnectData.Handle = 0;
                ConnectData.SingleAxis = null;
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            btnDisconnect.Enabled = false;
            rdbCAN.Checked = false;
            rdbEth.Checked = false;
            ConnectData.ToOutput("Disconnected.");
            ConnectData.Default();
        }

        private void GetVersionClick(object sender, EventArgs e)
        {
            UInt32 boot;
            String ver = string.Empty;
            try
            {
                MMCConnection.GetVersion(ConnectData.Handle, out boot, out ver);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("Boot version = " + boot.ToString() + " FW version = " + ver);
            string ver_ex = string.Empty;
            try
            {
                
                MMCConnection.GetVersionEx(ConnectData.Handle, out ver_ex);
            }
            catch( MMCException ex1 )
            {
                MessageBox.Show(ex1.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("Extended information: " + ver_ex);
        }

        private void btnAxisConnect_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.SingleAxis = new MMCSingleAxis(tbAxisName.Text, ConnectData.Handle);//create new object of MMCSingleAxis 
            }
            catch (MMCException ex)
            {
                ConnectData.SingleAxis = null;
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("Axis Reference of Axis " + tbAxisName.Text + " = " + ConnectData.SingleAxis.AxisReference.ToString() +
                                            ". DriveID of the Axis = " + ConnectData.SingleAxis.DriveID.ToString());//get properties  sa.AxisReference
        }

        private void btnGroupConnect_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.GroupAxis = new MMCGroupAxis(tbGroupName.Text, ConnectData.Handle);//create new object of MMCSingleAxis 
            }
            catch (MMCException ex)
            {
                ConnectData.GroupAxis = null;
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ClearGroupAxes();
            string str = tbGroupAxes.Text;
            int index = -1;
            while( (index = str.IndexOf(',') )!= -1)
            {
                ConnectData.AddGroupAxis( str.Substring(0, index));
                str = str.Substring(index + 1);
            }
            ConnectData.AddGroupAxis(str.Substring(0 ));
            ConnectData.ToOutput("Group Reference of Group " + tbGroupName.Text + " = " + ConnectData.GroupAxis.AxisReference.ToString());
        }

        private void OnCaptionEnter(object sender, EventArgs e)
        {
            ConnectData.Show( "Provide Connection functionality", System.Drawing.Color.Blue);
        }

        private void btnNetwork_Click(object sender, EventArgs e)
        {
            MMCNetwork mcn = new MMCNetwork( ConnectData.Handle );
            if (ConnectData != null && ConnectData.IsCANInternal && ConnectData.Handle != 0)
            {

                MmcNetworkInfo ni = new MmcNetworkInfo();
                try
                {
                    mcn.GetNetworkInfo(ref ni);
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    return;
                }
                String str = "Network Scan complete";
                ConnectData.ToOutput(str);
                str = "BaudRate =" + ni.BusBaud + "   Bus Type = " + ni.BusType + "   Active Nodes Count =" +
                      ni.NumOfActiveNodes;
                ConnectData.ToOutput(str);
                for (int i = 0; i < ni.NumOfActiveNodes; i++)
                {
                    str = "NodeID = " + ni.NodesInfo[i].ucNodeID +
                          "  NodeType = " + ni.NodesInfo[i].uiNodeType +
                          "  CommType = " + ni.NodesInfo[i].ucCommType +
                          "  ComState = " + ni.NodesInfo[i].ucNodeState;
                    ConnectData.ToOutput(str);
                }
            }
            else if (ConnectData != null && ConnectData.IsCANInternal== false && ConnectData.Handle != 0)
            {
                MmcCommstatistics cstat = new MmcCommstatistics();
                try
                {
                    mcn.GetNetworkInfo(ref cstat);
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    return;
                }
                String str = "Network Scan complete.Statistics:";
                ConnectData.ToOutput(str);
                str = "Master State = " + cstat.MasterState + " Slave Number = " + cstat.NumOfSlaves;
                ConnectData.ToOutput(str);
                MmcCommdiagnostics cd = new MmcCommdiagnostics();
                try
                {
                    mcn.GetCommDiagnostics(ref cd);
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    return;
                }
                for( int i = 0; i < cstat.NumOfSlaves; i++ )
                {
                    str = "Invalid Frames Port0=" + cd.DiagnosticsSlavesArr[i].InvalidFramesPort0 +
                          " Invalid Frames Port1=" + cd.DiagnosticsSlavesArr[i].InvalidFramesPort1 +
                          " Invalid Frames Port2=" + cd.DiagnosticsSlavesArr[i].InvalidFramesPort2 +
                          " Invalid Frames Port3=" +  cd.DiagnosticsSlavesArr[i].InvalidFramesPort3;
                    ConnectData.ToOutput(str);
                    str = "LostLinkErrorsPort0=" + cd.DiagnosticsSlavesArr[i].LostLinkErrorsPort0 +
                          " LostLinkErrorsPort1=" + cd.DiagnosticsSlavesArr[i].LostLinkErrorsPort1 +
                          " LostLinkErrorsPort2=" + cd.DiagnosticsSlavesArr[i].LostLinkErrorsPort2 +
                          " LostLinkErrorsPort3=" + cd.DiagnosticsSlavesArr[i].LostLinkErrorsPort3;
                    ConnectData.ToOutput(str);
                    str = "RXErrorsPort0=" + cd.DiagnosticsSlavesArr[i].RXErrorsPort0 +
                         " RXErrorsPort1=" + cd.DiagnosticsSlavesArr[i].RXErrorsPort1 +
                         " RXErrorsPort2=" + cd.DiagnosticsSlavesArr[i].RXErrorsPort2 +
                         " RXErrorsPort3=" + cd.DiagnosticsSlavesArr[i].RXErrorsPort3;
                    ConnectData.ToOutput(str);
                }
            }
        }

        private void EnableWarningsChanged(object sender, EventArgs e)
        {
            if(ConnectData.Handle == 0 )
                return;

            MMCConnection.EnableWarning(ConnectData.Handle, chbWarningsEnable.Checked);
        }   
    }

    public class UserEventData
    {
        public MC_FUNC_IDS FuncID;
        public short Dummy;
        public UInt32 FLong;
        public UInt32 SLong;
        public String StrData;

        public UserEventData()
        {
        }
        public void Init(byte[] data)
        {
            ushort fId;
            UInt32 l1, l2;


            Int32 index = 0;
            fId = SwappedParam.Swap(BitConverter.ToUInt16(data, index));
            index += sizeof(UInt16);
            SwappedParam sp = new SwappedParam();
            short _dummy = SwappedParam.Swap(BitConverter.ToInt16(data, index));
            index += sizeof(UInt16);
            l1 = SwappedParam.Swap(BitConverter.ToUInt32(data, index));
            //sp.SwapIn(data, index, out l1);
            index += sizeof(UInt32);
            l2 = SwappedParam.Swap(BitConverter.ToUInt32(data, index));
            //sp.SwapIn(data, index, out l2);
            index += sizeof(UInt32);
            if (data.Length - index > 0)
            {
                char[] cArr = new char[data.Length - index];
                for (int i = 0; i < cArr.Length; i++)
                    cArr[i] = (char)data[index + i];
                StrData = new string(cArr);
            }
            FuncID = (MC_FUNC_IDS)fId;
            Dummy = _dummy;
            FLong = l1;
            SLong = l2;
        }
    }
}
