﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class DS401Control : UserControl, IParentPanel
    {
        private MMCDS401Axis _ioDev;
        private MMCECATIO _ecatIODev;
        public DS401Control()
        {
            InitializeComponent();
        }
        private System.Windows.Forms.Panel panel;
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }

        public System.Windows.Forms.Panel GetPanel()
        {
            return panel;
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
            ConnectData_OnDataChange();
        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.Handle == 0 )
                EnableButtons(false);
            else
                EnableButtons(true);
            
        }

        private void EnableButtons(bool enable)
        {
            btnDevCreate.Enabled = enable;
            if( _ioDev != null && ConnectData.Handle != 0 )
            {
                btnWrite.Enabled = true;
                btnRead.Enabled = true;
            }
            else
            {
                btnWrite.Enabled = false;
                btnRead.Enabled = false;
            }
        }

        private void btnDevCreate_Click(object sender, EventArgs e)
        {
            try
            {
                _ioDev = new MMCDS401Axis(textBox1.Text, ConnectData.Handle);
                _ecatIODev = new MMCECATIO(textBox1.Text, ConnectData.Handle);
            }
            catch( MMCException ex )
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                _ioDev = null;
                _ecatIODev = null;
            }
            ConnectData_OnDataChange();
        }

        private void btnWrite_Click(object sender, EventArgs e)
        {
            UInt64 ulVal;
            if (UInt64.TryParse(tbWriteValue.Text, out ulVal))
            {
                try
                {
                    _ioDev.WriteDS401DOutput(0);
                }
                catch( MMCException ex )
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                }
            }
        }

        private void btnRead_Click(object sender, EventArgs e)
        {
            ulong rVal;
            try
            {
                rVal = _ioDev.ReadDS401DInput();
            }
            catch( MMCException ex )
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "DS401 Read completed. Accepted value = " + rVal.ToString("X");
            ConnectData.ToOutput(res);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            int erCode = Int32.Parse(tbErrCode.Text);
            OpenFileDialog dlg = new OpenFileDialog
            {
                Title = "Select Personality File",
                Filter = "Firmware (*.xml)|All Files (*.*)|*.*"
            };
            if (dlg.ShowDialog() != DialogResult.OK)
                return;
            String str = MMCConnection.GetErrStringFromPersonalityFileDrive(erCode, dlg.FileName);
            if (!String.IsNullOrEmpty(str))
            {
                ConnectData.ToOutput( str);
            }
        }

        private void btnReadAI_Click(object sender, EventArgs e)
        {
            try
            {
                int val = _ecatIODev.ECATIOReadAnalogInput(0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                _ecatIODev.ECATIOWriteAnalogOutput(0, 0xFF);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
            }
        }
    }
}
