﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;


namespace GMAS_Lib_test
{
    public partial class GroupAxisManagmentControl : UserControl, IParentPanel
    {
        public GroupAxisManagmentControl()
        {
            InitializeComponent();
        }

        private System.Windows.Forms.Panel panel;
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }
        
        public System.Windows.Forms.Panel GetPanel()
        {
            return panel;
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
            Array arDir = Enum.GetValues(typeof(MC_COORD_SYSTEM_ENUM));
            foreach (object obj in arDir)
                cmbCoordSystem.Items.Add(obj);

            Array arOPM = Enum.GetValues(typeof(MC_BUFFERED_MODE_ENUM));
            foreach (object obj in arOPM)
                cmbBuffMode.Items.Add(obj);

            Array TrParam = Enum.GetValues(typeof(NC_TRANSITION_MODE_ENUM));
            foreach (object obj in TrParam)
                cmbTransMode.Items.Add(obj);
            ConnectData_OnDataChange();
        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.GroupAxis == null)
                EnableButtons(false);
            else
            {
                EnableButtons(true);
                tbAcc.Text = ConnectData.GroupAxis.Acceleration.ToString();
                tbDec.Text = ConnectData.GroupAxis.Deceleration.ToString();
                tbJerk.Text = ConnectData.GroupAxis.Jerk.ToString();
                tbVelocity.Text = ConnectData.GroupAxis.Velocity.ToString();
                tbSuperimposed.Text = ConnectData.GroupAxis.Superimposed.ToString();

                cmbCoordSystem.SelectedItem = ConnectData.GroupAxis.CoordSystem;
                cmbBuffMode.SelectedItem = ConnectData.GroupAxis.GroupBufferedMode;
                cmbTransMode.SelectedItem = ConnectData.GroupAxis.TransitionMode;

                tbAuxPoint0.Text = ConnectData.GroupAxis.AuxPoint[0].ToString();
                tbAuxPoint1.Text = ConnectData.GroupAxis.AuxPoint[1].ToString();

                tbEndPoint0.Text = ConnectData.GroupAxis.EndPoint[0].ToString();
                tbEndPoint1.Text = ConnectData.GroupAxis.EndPoint[1].ToString();

                tbTransParam0.Text = ConnectData.GroupAxis.TransitionParameter[0].ToString();
                tbTransParam1.Text = ConnectData.GroupAxis.TransitionParameter[1].ToString();
            }
                

        }
        private void EnableButtons(bool enable)
        {
            tbAcc.Enabled = enable;
            tbDec.Enabled = enable;
            tbJerk.Enabled = enable;
            tbVelocity.Enabled = enable;
            tbSuperimposed.Enabled = enable;

            cmbCoordSystem.Enabled = enable;
            cmbBuffMode.Enabled = enable;
            cmbTransMode.Enabled = enable;

            tbAuxPoint0.Enabled = enable;
            tbAuxPoint1.Enabled = enable;

            tbEndPoint0.Enabled = enable;
            tbEndPoint1.Enabled = enable;

            tbTransParam0.Enabled = enable;
            tbTransParam1.Enabled = enable;
        }

        private void btnSet_Click(object sender, EventArgs e)
        {
            if (!Double.TryParse(tbAcc.Text, out ConnectData.GroupAxis.Acceleration))
                MessageBox.Show("Acceleration Value Error", "Argument Error");

            if (!Double.TryParse(tbDec.Text, out ConnectData.GroupAxis.Deceleration))
                MessageBox.Show("Deceleration Value Error", "Argument Error");

            if (!Double.TryParse(tbJerk.Text, out ConnectData.GroupAxis.Jerk))
                MessageBox.Show("Jerk Value Error", "Argument Error");

            if (!Double.TryParse(tbVelocity.Text, out ConnectData.GroupAxis.Velocity))
                MessageBox.Show("Velocity Value Error", "Argument Error");

            if (!Byte.TryParse(tbSuperimposed.Text, out ConnectData.GroupAxis.Superimposed))
                MessageBox.Show("Superimposed Value Error", "Argument Error");

            
            ConnectData.GroupAxis.CoordSystem = (MC_COORD_SYSTEM_ENUM)cmbCoordSystem.SelectedItem;
            ConnectData.GroupAxis.GroupBufferedMode = (MC_BUFFERED_MODE_ENUM)cmbBuffMode.SelectedItem;
            ConnectData.GroupAxis.TransitionMode = (NC_TRANSITION_MODE_ENUM)cmbTransMode.SelectedItem;

            if (!Double.TryParse(tbAuxPoint0.Text, out ConnectData.GroupAxis.AuxPoint[0]))
                MessageBox.Show("AuxPoint[0] Value Error", "Argument Error");

            if (!Double.TryParse(tbAuxPoint1.Text, out ConnectData.GroupAxis.AuxPoint[1]))
                MessageBox.Show("AuxPoint[1] Value Error", "Argument Error");

            if (!Double.TryParse(tbEndPoint0.Text, out ConnectData.GroupAxis.EndPoint[0]))
                MessageBox.Show("EndPoint[0] Value Error", "Argument Error");

            if (!Double.TryParse(tbEndPoint1.Text, out ConnectData.GroupAxis.EndPoint[1]))
                MessageBox.Show("EndPoint[1] Value Error", "Argument Error");

            if (!Double.TryParse(tbTransParam0.Text, out ConnectData.GroupAxis.TransitionParameter[0]))
                MessageBox.Show("TransParam[0] Value Error", "Argument Error");

            if (!Double.TryParse(tbTransParam1.Text, out ConnectData.GroupAxis.TransitionParameter[1]))
                MessageBox.Show("TransParam[1] Value Error", "Argument Error");
        }

    }
}
