﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class GroupMovingControl : UserControl, IParentPanel
    {
        public GroupMovingControl()
        {
            InitializeComponent();
        }

        private System.Windows.Forms.Panel panel;
        private MMC_MOTIONPARAMS_GROUP grParam;
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }
        public System.Windows.Forms.Panel GetPanel()
        {
            return panel;
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            if (ConnectData.IsValid == false || ConnectData.GroupAxis == null)
                EnableButtons(false);
            else
            {
                EnableButtons(true);
                btnSetDefParams_Click( this, null);
            }
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);

        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.GroupAxis == null)
                EnableButtons(false);
            else
                EnableButtons(true);

        }
        private void EnableButtons(bool enable)
        {
            btnDisable.Enabled = enable;
            btnEnable.Enabled = enable;
            btnSetKinTransform.Enabled = enable;
            btnMoveCircAbs.Enabled = enable;
            btnMoveLineAbs.Enabled = enable;
            btnMoveRel.Enabled = enable;
            btnInitEx.Enabled = enable;
            btnStop.Enabled = enable;
            btnHalt.Enabled = enable;
            btnSetDefParams.Enabled = enable;
            btnGo.Enabled = enable; 
        }

        private void btnEnable_Click(object sender, EventArgs e)
        {
            byte cnt = 0;
            MMCSingleAxis[] arr = ConnectData.GetGroupAxes( out cnt);
            if (cnt <= 0 || arr == null)
                return;
            foreach (MMCSingleAxis mmcSingleAxise in arr)
            {
                if (mmcSingleAxise != null)
                {
                    try
                    {
                        mmcSingleAxise.PowerOn(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
                    }
                    catch (MMCException ex)
                    {
                        MessageBox.Show(ex.ToString(), "MMC Exception");

                    }
                }
            }
            System.Threading.Thread.Sleep( 2000 );
            try
            {
                ConnectData.GroupAxis.GroupEnable();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            ConnectData.ToOutput("GroupEnable command accepted  for group " + ConnectData.GroupAxis.AxisName);
        }

        private void btnDisable_Click(object sender, EventArgs e)
        {
            byte cnt = 0;
            MMCSingleAxis[] arr = ConnectData.GetGroupAxes(out cnt);
            if (cnt <= 0 || arr == null)
                return;
            foreach (MMCSingleAxis mmcSingleAxise in arr)
            {
                if (mmcSingleAxise != null)
                {
                    try
                    {
                        mmcSingleAxise.PowerOff(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
                    }
                    catch (MMCException ex)
                    {
                        MessageBox.Show(ex.ToString(), "MMC Exception");

                    }
                }
            }

            try
            {
                ConnectData.GroupAxis.GroupDisable();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            ConnectData.ToOutput("GroupDisable command accepted  for group " + ConnectData.GroupAxis.AxisName);
        }

        private void btnMoveCircAbs_Click(object sender, EventArgs e)
        {
           
            //ConnectData.GroupAxis.AuxPoint[0] = 10000;
            //ConnectData.GroupAxis.AuxPoint[1] = 10000;
            //ConnectData.GroupAxis.CoordSystem = MC_COORD_SYSTEM_ENUM.MC_MCS_COORD;
            //ConnectData.GroupAxis.TransitionParameter[0] = 1;
            //ConnectData.GroupAxis.TransitionParameter[1] = 1;
            //ConnectData.GroupAxis.Jerk = 2000000;
            //ConnectData.GroupAxis.Acceleration = 1000;
            //ConnectData.GroupAxis.Deceleration = 1000;
            try
            {
                ConnectData.GroupAxis.MoveCircularAbsolute(NC_ARC_SHORT_LONG_ENUM.MC_NONE_ARC_CHOICE,
                                                           NC_PATH_CHOICE_ENUM.MC_CLOCKWISE,
                                                           NC_CIRC_MODE_ENUM.MC_BORDER_CIRC_MODE,
                                                           ConnectData.GroupAxis.AuxPoint,
                                                           ConnectData.GroupAxis.EndPoint,
                                                           ConnectData.GroupAxis.GroupBufferedMode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("MoveCircularAbsolute command accepted  for group " + ConnectData.GroupAxis.AxisName);
        }

        private void btnMoveLineAbs_Click(object sender, EventArgs e)
        {
           
            //ConnectData.GroupAxis.AuxPoint[0] = 15000;
            //ConnectData.GroupAxis.AuxPoint[1] = 12000;
            //ConnectData.GroupAxis.Jerk = 2000000;
            //ConnectData.GroupAxis.Acceleration = 1000;
            //ConnectData.GroupAxis.Deceleration = 1000;
            //ConnectData.GroupAxis.CoordSystem = MC_COORD_SYSTEM_ENUM.MC_ACS_COORD;
            //ConnectData.GroupAxis.TransitionMode = NC_TRANSITION_MODE_ENUM.MC_TM_NONE_MODE;
            //ConnectData.GroupAxis.TransitionParameter[1] = 2;
            //ConnectData.GroupAxis.TransitionParameter[2] = 1;
            //ConnectData.GroupAxis.TransitionParameter[3] = 1;
            //ConnectData.GroupAxis.TransitionParameter[4] = 1;
            //ConnectData.GroupAxis.Superimposed = 0;
            ConnectData.GroupAxis.Execute = 1;
            try
            {
                ConnectData.GroupAxis.MoveLinearAbsolute(   (float) ConnectData.GroupAxis.Velocity,
                                                            ConnectData.GroupAxis.EndPoint,
                                                            ConnectData.GroupAxis.GroupBufferedMode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("MoveLinearAbsolute command accepted  for group " + ConnectData.GroupAxis.AxisName);
        }

        private void btnMoveRel_Click(object sender, EventArgs e)
        {
            //ConnectData.GroupAxis.AuxPoint[0] = 15000;
            //ConnectData.GroupAxis.AuxPoint[1] = 12000;
            //ConnectData.GroupAxis.Jerk = 2000000;
            //ConnectData.GroupAxis.Acceleration = 1000;
            //ConnectData.GroupAxis.Deceleration = 1000;
            //ConnectData.GroupAxis.CoordSystem = MC_COORD_SYSTEM_ENUM.MC_ACS_COORD;
            //ConnectData.GroupAxis.TransitionMode = NC_TRANSITION_MODE_ENUM.MC_TM_NONE_MODE;
            //ConnectData.GroupAxis.TransitionParameter[1] = 2;
            //ConnectData.GroupAxis.TransitionParameter[2] = 1;
            //ConnectData.GroupAxis.TransitionParameter[3] = 1;
            //ConnectData.GroupAxis.TransitionParameter[4] = 1;
            //ConnectData.GroupAxis.Superimposed = 0;
            //ConnectData.GroupAxis.Velocity = 10000;
            ConnectData.GroupAxis.Execute = 1;
            try
            {
                ConnectData.GroupAxis.MoveLinearRelative(   (float) ConnectData.GroupAxis.Velocity,
                                                            ConnectData.GroupAxis.EndPoint,
                                                            ConnectData.GroupAxis.GroupBufferedMode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("MoveLinearRelative command accepted  for group " + ConnectData.GroupAxis.AxisName);
        }

        private void btnSetKinTransform_Click(object sender, EventArgs e)
        {
            byte count = 0;
            MMCSingleAxis[] AxArr = this.ConnectData.GetGroupAxes(out count);
            if (count <= 0 || AxArr == null)
                return;
            if (radioButtonCartesian.Checked)
            {
                MC_KIN_REF_CARTESIAN inputC = new MC_KIN_REF_CARTESIAN();
                inputC.iNumAxes = 3;

                inputC.sNode[0].eType = NC_AXIS_IN_GROUP_TYPE_ENUM_EX.NC_PROFILER_X_AXIS_TYPE;
                inputC.sNode[0].hNode = AxArr[0].AxisReference;
                inputC.sNode[0].iMcsToAcsFuncID = NC_TR_FUNC_ID_ENUM.NC_TR_SHIFT_FUNC;
                inputC.sNode[0].ulTrCoef[0] = 100.0;
                inputC.sNode[0].ulTrCoef[1] = 1.0/(100.0);
                inputC.sNode[0].ulTrCoef[2] = 22;

                inputC.sNode[1].eType = NC_AXIS_IN_GROUP_TYPE_ENUM_EX.NC_PROFILER_Y_AXIS_TYPE;
                inputC.sNode[1].hNode = AxArr[1].AxisReference;
                inputC.sNode[1].iMcsToAcsFuncID = NC_TR_FUNC_ID_ENUM.NC_TR_SHIFT_FUNC;
                inputC.sNode[1].ulTrCoef[0] = 100.0;
                inputC.sNode[1].ulTrCoef[1] = 1.0/(100.0);
                inputC.sNode[1].ulTrCoef[2] = 22;

                inputC.sNode[2].eType = NC_AXIS_IN_GROUP_TYPE_ENUM_EX.NC_PROFILER_Z_AXIS_TYPE;
                inputC.sNode[2].hNode = AxArr[2].AxisReference;
                inputC.sNode[2].iMcsToAcsFuncID = NC_TR_FUNC_ID_ENUM.NC_TR_SHIFT_FUNC;
                inputC.sNode[2].ulTrCoef[0] = 100.0;
                inputC.sNode[2].ulTrCoef[1] = 1.0/(100.0);
                inputC.sNode[2].ulTrCoef[2] = 22;

                try
                {
                    ConnectData.GroupAxis.SetKinTransformExCartesian(inputC);
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    return;
                }
                return;
            }
            if (radioButtonDelta.Checked)
            {
                MC_KIN_REF_DELTA input1 = new MC_KIN_REF_DELTA();
                input1.dbArm = 160000;
                input1.dbForeArm = 320000;
                input1.dbBaseRadius = 50000;
                input1.dbEndEffectorRadius = 40000;


                input1.iNumAxes = 3;

                input1.sNode[0].eType = NC_AXIS_IN_GROUP_TYPE_ENUM_EX.NC_ACS_A1_AXIS_TYPE;
                input1.sNode[0].hNode = AxArr[0].AxisReference;
                input1.sNode[0].iMcsToAcsFuncID = NC_TR_FUNC_ID_ENUM.NC_TR_SHIFT_FUNC;
                input1.sNode[0].ulTrCoef[0] = -131072.0;
                input1.sNode[0].ulTrCoef[1] = -1.0/(131072.0);
                input1.sNode[0].ulTrCoef[2] = 16600;

                input1.sNode[1].eType = NC_AXIS_IN_GROUP_TYPE_ENUM_EX.NC_ACS_A2_AXIS_TYPE;
                input1.sNode[1].hNode = AxArr[1].AxisReference;
                input1.sNode[1].iMcsToAcsFuncID = NC_TR_FUNC_ID_ENUM.NC_TR_SHIFT_FUNC;
                input1.sNode[1].ulTrCoef[0] = -131072.0;
                input1.sNode[1].ulTrCoef[1] = -1.0/(131072.0);
                input1.sNode[1].ulTrCoef[2] = 16700;

                input1.sNode[2].eType = NC_AXIS_IN_GROUP_TYPE_ENUM_EX.NC_ACS_A3_AXIS_TYPE;
                input1.sNode[2].hNode = AxArr[2].AxisReference;
                input1.sNode[2].iMcsToAcsFuncID = NC_TR_FUNC_ID_ENUM.NC_TR_SHIFT_FUNC;
                input1.sNode[2].ulTrCoef[0] = -131072.0;
                input1.sNode[2].ulTrCoef[1] = -1.0/(131072.0);
                input1.sNode[2].ulTrCoef[2] = 16100;
                try
                {
                    ConnectData.GroupAxis.SetKinTransformExDeltaRobot(input1);
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    return;
                }

                return;
            }
            MmcSetkintransformParam ktf = new MmcSetkintransformParam();
            ktf.NcType[0] = NC_AXIS_IN_GROUP_TYPE_ENUM.NC_X_AXIS_TYPE;
            ktf.NcType[1] = NC_AXIS_IN_GROUP_TYPE_ENUM.NC_Y_AXIS_TYPE;
            ktf.NcType[2] = NC_AXIS_IN_GROUP_TYPE_ENUM.NC_Z_AXIS_TYPE;
            ktf.NcType[3] = NC_AXIS_IN_GROUP_TYPE_ENUM.NC_N1_AXIS_TYPE;
            ktf.NcType[4] = NC_AXIS_IN_GROUP_TYPE_ENUM.NC_N2_AXIS_TYPE;

            ktf.Node[1] = 1;
            ktf.Node[2] = 2;
            ktf.Node[3] = 3;
            ktf.Node[4] = 4;

            ktf.McsToAcsFuncID[0] = NC_TR_FUNC_ID_ENUM.NC_TR_SHIFT_FUNC;
            ktf.McsToAcsFuncID[1] = NC_TR_FUNC_ID_ENUM.NC_TR_SHIFT_FUNC;
            ktf.McsToAcsFuncID[2] = NC_TR_FUNC_ID_ENUM.NC_TR_SHIFT_FUNC;
            ktf.McsToAcsFuncID[3] = NC_TR_FUNC_ID_ENUM.NC_TR_SHIFT_FUNC;
            ktf.McsToAcsFuncID[4] = NC_TR_FUNC_ID_ENUM.NC_TR_SHIFT_FUNC;

            ktf.NumAxes = 2;
            ktf.BufferMode = MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE;
            ktf.TrCoef[0] = 1;
            ktf.TrCoef[1] = 1;
            ktf.TrCoef[3] = 0;

            ktf.TrCoef[4] = 1;
            ktf.TrCoef[5] = 1;
            ktf.TrCoef[6] = 0;

            ktf.TrCoef[7] = 1;
            ktf.TrCoef[8] = 1;
            ktf.TrCoef[9] = 0;

            ktf.TrCoef[10] = 1;
            ktf.TrCoef[11] = 1;
            ktf.TrCoef[12] = 0;

            ktf.TrCoef[13] = 1;
            ktf.TrCoef[14] = 1;
            ktf.TrCoef[15] = 0;

            try
            {
                ConnectData.GroupAxis.SetKinTransform(ref ktf);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("SetKinTransform command accepted  for group " + ConnectData.GroupAxis.AxisName);
        }

        private void btnInitExample_click(object sender, EventArgs e)
        {
            btnDisable_Click(this, null);
            byte cnt = 0;
            MMCSingleAxis[] arr = ConnectData.GetGroupAxes(out cnt);
            if (cnt <= 0 || arr == null)
                return;
            int param = 2000;
            byte[] cmdArr = new byte[2];
            cmdArr[0] = (byte)'p';
            cmdArr[1] = (byte)'x';
            foreach (MMCSingleAxis mmcSingleAxise in arr)
            {
                if (mmcSingleAxise != null)
                {
                    param += 2000;
                    try
                    {
                        mmcSingleAxise.ElmoSetAsyncParam(cmdArr, ref param);
                    }
                    catch (MMCException ex)
                    {
                        MessageBox.Show(ex.ToString(), "MMC Exception");

                    }
                }
            }
            ConnectData.ToOutput("Example was initialized for group " + ConnectData.GroupAxis.AxisName);
        }

        private void btnHalt_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.GroupAxis.GroupHalt(1000, 2000000, MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("GroupHalt command accepted  for group " + ConnectData.GroupAxis.AxisName);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.GroupAxis.GroupStop(1000, 2000000, MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("GroupStop command accepted  for group " + ConnectData.GroupAxis.AxisName);
        }

        private void btnSetDefParams_Click(object sender, EventArgs e)
        {
            if( grParam == null )
                grParam = new MMC_MOTIONPARAMS_GROUP();
            grParam.fVelocity = 10000;
            grParam.fAcceleration = 1000;
            grParam.fDeceleration = 1000;
            grParam.eTransitionMode = NC_TRANSITION_MODE_ENUM.MC_TM_NONE_MODE;
            grParam.eCoordSystem = MC_COORD_SYSTEM_ENUM.MC_ACS_COORD;
            grParam.eBufferMode = MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE;
            grParam.iExecDelayMs = 0;
            grParam.fJerk = 1000000;
            grParam.bExecute = 1;
            
            float[] tParam = new float[16];
            int i = 0;
            for( i = 0; i < 16; i++)
            {
                if( i < 2 )
                    tParam[i] = 1;
                else
                    tParam[i] = 0;
            }
            tParam.CopyTo( grParam.fTransitionParameter, 0); 

            double[] auxPoint = new double[16]; 
            for( i = 0; i < 16; i++)
            {
                switch( i )
                {
                    case 0:
                        auxPoint[i] = 5000;
                        break;
                    case 1:
                        auxPoint[i] = auxPoint[i - 1]*2;
                        break;
                    default:
                        auxPoint[i] = 0;
                        break;
                }
                
            }
            auxPoint.CopyTo(grParam.dAuxPoint, 0);

            double[] endPoint = new double[16];
            for (i = 0; i < 16; i++)
            {
                switch (i)
                {
                    case 0:
                        endPoint[i] = 5000;
                        break;
                    case 1:
                        endPoint[i] = endPoint[i - 1] * 2;
                        break;
                    default:
                        endPoint[i] = 0;
                        break;
                }

            }
            endPoint.CopyTo(grParam.dEndPoint, 0);

            ConnectData.GroupAxis.SetDefaultParams( grParam );
        }

        private void btnGo_Click(object sender, EventArgs e)
        {
            if( grParam == null )
                btnSetDefParams_Click( this, null);

            ConnectData.GroupAxis.SetDefaultParams( grParam );
            try
            {
                ConnectData.GroupAxis.MoveLinearRelative((float) ConnectData.GroupAxis.Velocity,
                                                            ConnectData.GroupAxis.EndPoint,
                                                            MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            grParam.dEndPoint[0] *= -1;
            grParam.dEndPoint[1] *= -1;
            grParam.fVelocity *= 10;
            grParam.fAcceleration *= 10;
            grParam.fDeceleration *= 10;

            ConnectData.GroupAxis.SetDefaultParams(grParam);
            try
            {
                ConnectData.GroupAxis.MoveLinearRelative((float) ConnectData.GroupAxis.Velocity,
                                                            ConnectData.GroupAxis.EndPoint,
                                                            MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

        }

        private void label1_Click(object sender, EventArgs e)
        {

        }

        private void btnMoveAdditive_Click(object sender, EventArgs e)
        {
            ConnectData.GroupAxis.Execute = 1;
            try
            {
                ConnectData.GroupAxis.MoveLinearAdditive((float) ConnectData.GroupAxis.Velocity,
                                                            ConnectData.GroupAxis.EndPoint,
                                                            ConnectData.GroupAxis.GroupBufferedMode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("MoveLinearAdditive command accepted  for group " + ConnectData.GroupAxis.AxisName);
        }
    }
}
