﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class HomingControl : UserControl, IParentPanel
    {
        public HomingControl()
        {
            InitializeComponent();
            Visible = false;
            hParam = new HomeParams();
        }
        private System.Windows.Forms.Panel panel;
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }
        public HomeParams HomingParam
        {
            get { return hParam; }
        }
        private HomeParams hParam; //{ get; private set; }
        public System.Windows.Forms.Panel GetPanel()
        {
            return panel;
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
            ConnectData_OnDataChange();
        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.SingleAxis == null)
                EnableButtons(false);
            else
                EnableButtons(true);
            
        }
        private void EnableButtons(bool enable)
        {
            btnPowerOn.Enabled = enable;
            btnPowerOff.Enabled = enable;
            btnStop.Enabled = enable;
            btnHome.Enabled = enable;
            btnSetMode.Enabled = enable;
        }

        private void OnLoad(object sender, EventArgs e)
        {
            Visible = true;
            Dock = DockStyle.Fill;
            Array arOPM = Enum.GetValues(typeof(MC_BUFFERED_MODE_ENUM));
            foreach (object obj in arOPM)
                cmbBuffMode.Items.Add(obj);
            cmbBuffMode.SelectedIndex = 2;

            Array arDir = Enum.GetValues(typeof(MC_HOME_DIRECTION_ENUM));
            foreach (object obj in arDir)
                cmbDir.Items.Add(obj);
            cmbDir.SelectedIndex = 0;

            Array arSwM = Enum.GetValues(typeof(MC_SWITCH_MODE_ENUM));
            foreach (object obj in arSwM)
                cmbSwitchMode.Items.Add(obj);
            cmbSwitchMode.SelectedIndex = 0;

            Array arHM = Enum.GetValues(typeof(MMC_HOME_MODE_ENUM));
            foreach (object obj in arHM)
                cmbHomeMode.Items.Add(obj);
            cmbHomeMode.SelectedIndex = 0;
            OnVisibleChange( sender, e);
        }

        private bool MotionPreview()
        {
            MC_STATE_SINGLE state = 0;
            uint plcStateMask = 0x7F8;//0x3F8;//0x7;//0x00000400
            uint validBitMask = 0x40000000;//0x3F8;//0x7;//0x00000400

            UInt32 rv = 0;

            try
            {
                rv = ConnectData.SingleAxis.ReadStatus();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                EnableButtons(false);
                MessageBox.Show("Please disconnect and reboot device", "Internal Communication Error");
                ConnectData.Default();
                return false;
            }

            if ((rv & validBitMask) == validBitMask)
            {
                state = (MC_STATE_SINGLE)(rv & plcStateMask);
            }

            if ((state & MC_STATE_SINGLE.ERROR_STOP) == MC_STATE_SINGLE.ERROR_STOP)
            {
                try
                {
                    ConnectData.SingleAxis.Reset();
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    EnableButtons(false);
                    MessageBox.Show("Please disconnect and reboot device", "Internal Communication Error");
                    ConnectData.Default();
                    return false;
                }
            }

            try
            {
                rv = ConnectData.SingleAxis.ReadStatus();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                EnableButtons(false);
                MessageBox.Show("Please disconnect and reboot device", "Internal Communication Error");
                ConnectData.Default();
                return false;
            }
            if ((rv & validBitMask) == validBitMask)
            {
                state = (MC_STATE_SINGLE)(rv & plcStateMask);
            }
            else
            {
                MessageBox.Show("Illegal state recognized .Possible Reason - GMAS is in Preoperational mode.",
                                "Homing Control");
                //ConnectData.Default();
                return false;
            }

            return true;
        }

        private void OnVisibleChange(object sender, EventArgs e)
        {
            if (Visible == false)
            {
                if (ConnectData != null && ConnectData.IsValid && ConnectData.SingleAxis != null)
                {
                    try
                    {
                        ConnectData.SingleAxis.PowerOff(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
                    }
                    catch (MMCException ex)
                    {
                        MessageBox.Show(ex.ToString(), "MMC Exception");
                        return;
                    }
                }
            }
            else
            {
                if (ConnectData != null && ConnectData.IsValid && ConnectData.SingleAxis != null)
                {
                    hParam.dbPosition = ConnectData.SingleAxis.HomePosition;
                    hParam.eBufferMode = ConnectData.SingleAxis.HomeBufferedMode;
                    hParam.eDirection = ConnectData.SingleAxis.HomeDirection;
                    hParam.eHomingMode = ConnectData.SingleAxis.HomingMode;
                    hParam.eSwitchMode = ConnectData.SingleAxis.HomeSwitchMode;
                    hParam.fAcceleration = (float) ConnectData.SingleAxis.HomeAcceleration;
                    hParam.fDistanceLimit = (float) ConnectData.SingleAxis.HomeDistanceLimit;
                    hParam.fTorqueLimit = (float) ConnectData.SingleAxis.HomeTorqueLimit;
                    hParam.fVelocity = (float) ConnectData.SingleAxis.HomeVelocity;
                    hParam.ucExecute = ConnectData.SingleAxis.HomeExecute;
                    hParam.uiTimeLimit = ConnectData.SingleAxis.HomeTimeLimit;

                    tbHPos.Text = HomingParam.dbPosition.ToString();
                    tbAcc.Text = HomingParam.fAcceleration.ToString();
                    tbDistanceLimit.Text = HomingParam.fDistanceLimit.ToString();
                    tbTimeLimit.Text = HomingParam.uiTimeLimit.ToString();
                    tbTorqLimit.Text = HomingParam.fTorqueLimit.ToString();
                    tbVelocity.Text = HomingParam.fVelocity.ToString();

                    cmbBuffMode.SelectedItem = HomingParam.eBufferMode;
                    cmbDir.SelectedItem = HomingParam.eDirection;
                    cmbHomeMode.SelectedItem = HomingParam.eHomingMode;
                    cmbSwitchMode.SelectedItem = HomingParam.eSwitchMode;
                    
                    //MotionPreview();
                }
            }
        }

        private void btnPowerOn_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.SingleAxis.PowerOn(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "PowerOn command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnPowerOff_Click(object sender, EventArgs e)
        {
            try
            {
                try
                {
                    ConnectData.SingleAxis.PowerOff(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    return;
                }
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "PowerOff command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            if (!MotionPreview())
                return;
            try
            {
                ConnectData.SingleAxis.Stop(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Stop command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

        private void btnHome_Click(object sender, EventArgs e)
        {

            if (!MotionPreview())
                return;

            if (tbHPos.Text == null || tbHPos.Text == string.Empty)
                hParam.dbPosition = ConnectData.SingleAxis.HomePosition;
            else
            {
                if (!Double.TryParse(tbHPos.Text, out hParam.dbPosition))
                    MessageBox.Show("Position Value Error", "Argument Error");
            }
            hParam.eBufferMode = (MC_BUFFERED_MODE_ENUM)cmbBuffMode.SelectedItem ;
            hParam.eDirection = (MC_HOME_DIRECTION_ENUM) cmbDir.SelectedItem;
            hParam.eHomingMode = (MMC_HOME_MODE_ENUM) cmbHomeMode.SelectedItem;
            hParam.eSwitchMode = (MC_SWITCH_MODE_ENUM)cmbSwitchMode.SelectedItem;

            if (tbAcc.Text == null || tbAcc.Text == string.Empty)
                hParam.fAcceleration = (float) ConnectData.SingleAxis.HomeAcceleration;
            else
            {
                if (!Single.TryParse(tbAcc.Text, out hParam.fAcceleration))
                    MessageBox.Show("Acceleration Value Error", "Argument Error");
            }

            if (tbDistanceLimit.Text == null || tbDistanceLimit.Text == string.Empty)
                hParam.fDistanceLimit = (float) ConnectData.SingleAxis.HomeDistanceLimit;
            else
            {
                if (!Single.TryParse(tbDistanceLimit.Text, out hParam.fAcceleration))
                    MessageBox.Show("Distance Value Error", "Argument Error");
            }

            if (tbTimeLimit.Text == null || tbTimeLimit.Text == string.Empty)
                hParam.uiTimeLimit = ConnectData.SingleAxis.HomeTimeLimit;
            else
            {
                if (!UInt32.TryParse(tbTimeLimit.Text, out hParam.uiTimeLimit))
                    MessageBox.Show("Time Limit Value Error", "Argument Error");
            }

            if (tbTorqLimit.Text == null || tbTorqLimit.Text == string.Empty)
                hParam.fTorqueLimit = (float) ConnectData.SingleAxis.HomeTorqueLimit;
            else
            {
                if (!Single.TryParse(tbTorqLimit.Text, out hParam.fTorqueLimit))
                    MessageBox.Show("Torque Limit Value Error", "Argument Error");
            }

            if (tbVelocity.Text == null || tbVelocity.Text == string.Empty)
                hParam.fVelocity = (float) ConnectData.SingleAxis.HomeVelocity;
            else
            {
                if (!Single.TryParse(tbVelocity.Text, out hParam.fVelocity))
                    MessageBox.Show("Velocity Limit Value Error", "Argument Error");    
            }

            short errID = 0;
            UInt32 fbHandle = 0;
            try
            {
                ConnectData.SingleAxis.Home(hParam,ref errID, ref fbHandle);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Homing command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

       

        private void btnSetMode_Click(object sender, EventArgs e)
        {
            OPM402 opMode = ConnectData.SingleAxis.GetOpMode();
            if (opMode != OPM402.OPM402_HOMING_MODE)
            {
                try
                {
                    ConnectData.SingleAxis.SetOpMode(OPM402.OPM402_HOMING_MODE);
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "Homing Control");
                }
                String res = "Operated Mode successfully changed to " + OPM402.OPM402_HOMING_MODE.ToString();
                ConnectData.ToOutput(res);
                return;
            }
            String rs = "Operated Mode already equal to " + OPM402.OPM402_HOMING_MODE.ToString();
            ConnectData.ToOutput(rs);
        }

    }
}
