﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class LoadPanel : UserControl, IParentPanel
    {
        public LoadPanel()
        {
            InitializeComponent();
        }

        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }
        ConnectivityData connectData;

        public Panel GetPanel()
        {
            return panel;
        }
        private Panel panel;
       
        public void Init(ConnectivityData data, Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += ConnectData_OnDataChange;
            ConnectData_OnDataChange();
        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.Handle == 0)
            {
                EnableButtons(false);
            }
            else
            {
                EnableButtons(true);
                LoadTable();
            }
        }

        private void LoadTable()
        {
            camTable = new MMCCamTable(ConnectData.Handle);
            camTable.UnloadTable(0xffffffff); //Unload all tables
        }

        private void EnableButtons( bool val )
        {
            btnLoad.Enabled = val;
            btnOpen.Enabled = val;
            comboLoadTipes.Enabled = val;
            buttonConnectToMaster.Enabled = val;
            buttonUnselectAll.Enabled = val;
            buttonInitTable.Enabled = val;
            buttonSelectTable.Enabled = val;
            buttonCamIn.Enabled = val && ConnectData.SingleAxis != null && axisMaster != null;
            buttonCamOut.Enabled = val && ConnectData.SingleAxis != null && axisMaster != null;
            buttonCamStatus.Enabled = val && ConnectData.SingleAxis != null && axisMaster != null;
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.AddExtension = true;
            dlg.CheckFileExists = true;
            dlg.CheckPathExists = true;
            dlg.AutoUpgradeEnabled = false;
            if (dlg.ShowDialog(null) != DialogResult.OK)
            {
                return;
            }
            tbPathTo.Text = dlg.FileName;
        }

        private DateTime start, fin;
        public void FinishEvent(int comHandle, LoadType type, Error error)
        {
            fin = DateTime.Now;
            Int64 deltaSec = fin.Second - start.Second;
            Int64 deltaMSec = fin.Millisecond - start.Millisecond;
            Int64 delta = deltaSec*1000 + deltaMSec;
            if (error != Error.OK)
            {
                MessageBox.Show(string.Format("Finished with error: {0} Timeout: {1}", error, delta));
            }
            else
            {
                MessageBox.Show("Successfully Finished");
            }
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            string txt = comboLoadTipes.SelectedItem as string;
            switch (txt)
            {
                case "USER_FILE":
                    start = DateTime.Now;
                    MMCConnection.DownloadUserFile(ConnectData.Handle, tbPathTo.Text, null, null,
                                                   FinishEvent);
                    break;
                default:
                    break;
            }
        }

        #region CAM Data

        private MMCSingleAxis axisMaster;
        private MMCCamTable camTable;
        private UInt32 camTableHandle = 0xFFFFFFFF;
        private const int PointsNum = 10;

        private readonly double[] TableData = new double[PointsNum*2]
            {
                0.0000000, 0.000000,
                10000.000000, 35000.000000,
                13000.000000, 60000.000000,
                15000.000000, 50000.000000,
                20000.000000, 80000.000000,
                44000.000000, 110000.000000,
                75000.000000, 122000.000000,
                103000.000000, 145000.000000,
                117000.000000, 160000.000000,
                120000.000000, 185000.000000
            };


        #endregion

        private void buttonConnectToMaster_Click(object sender, EventArgs e)
        {
            try
            {
                axisMaster = new MMCSingleAxis(textBoxMaster.Text, ConnectData.Handle);//create new object of MMCSingleAxis 
                EnableButtons(ConnectData.Handle != 0);
            }
            catch (MMCException ex)
            {
                axisMaster = null;
                MessageBox.Show(ex.ToString(), "MMC Exception");
            }
        }

        private void buttonInitTable_Click(object sender, EventArgs e)
        {
            try
            {
                Int16 errVal = camTable.InitTable(
                    MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE,
                    PointsNum, //Maximal number of pints for the CAM Table
                    2, //Underflow Treshold for the CAM Table
                    CURVE_TYPE_ENUM.eQuinticInterp, //CAM table Interpolation Type 
                    1, //Number of slaves. Since group is not supported on this phase, dimension, for the time being is always one
                    false, //Is the CAM Table in Superimposed mode
                    true, //Is the CAM Table has fixed Gap
                    true, //Execute FB
                    out camTableHandle); //CAM Table Handler
                if (errVal != 0)
                {
                    ConnectData.ToOutput(string.Format("InitTable return error {0}", errVal));
                    return;
                }
                ConnectData.ToOutput(string.Format("InitTable - OK! CAM Table Handler = {0}", camTableHandle));
                
                errVal = camTable.SelectTable(
                    (int) camTableHandle, // CAM Table Handler was retrieved by InitTable(). If < 0 means load new table from file. Otherwise it is an handle of existing table.
                    "", //CAM Table Path. Is ignored if invoqed InitTable()  
                    1000.0, //CAM Table Gap - relevant only on fixed gap
                    0.0, //Master Start Position - relevant only on absolute master and fixed gap
                    MC_COORD_SYSTEM_ENUM.MC_NONE_COORD,
                    CAM_TABLE_MODE_ENUM.eCAMT_RWMode, //Changeable CAM Table
                    0, //Master and Slave Start Mode. We will use bitwise: Bit 0 - Master Abs/Rel, Bit 1 - Slave Abs/Rel.
                    true, //Master position is Abs/Rel
                    true, //Slave position is Abs/Rel
                    out camTableHandle); //CAM Table Handler
                if (errVal != 0)
                {
                    ConnectData.ToOutput(string.Format("SelectTable return error {0}", errVal));
                    return;
                }
                ConnectData.ToOutput(string.Format("SelectTable - OK! CAM Table Handler = {0}", camTableHandle));

                errVal = camTable.AddTable(
                    camTableHandle, //CAM Table Handler
                    TableData, //List of CAM Table rows
                    0, //Row index to start from
                    PointsNum); //Number of rows to add to the CAM Table
                if (errVal != 0)
                {
                    ConnectData.ToOutput(string.Format("Add Table return error {0}", errVal));
                    return;
                }
                ConnectData.ToOutput("Add to Table - OK!");
            }
            catch (MMCException ex)
            {
                ConnectData.ToOutput(ex.What);
            }
        }

        private void buttonSelectTable_Click(object sender, EventArgs e)
        {
            try
            {
                Int16 errVal = camTable.SelectTable(
                    -1, // CAM Table Handler was retrieved by InitTable(). If < 0 means load new table from file. Otherwise it is an handle of existing table.
                    "/mnt/jffs/usr/table.cam", //CAM Table Path. Is ignored if invoqed InitTable()  
                    1000.0, //CAM Table Gap - relevant only on fixed gap
                    0.0, //Master Start Position - relevant only on absolute master and fixed gap
                    MC_COORD_SYSTEM_ENUM.MC_NONE_COORD,
                    CAM_TABLE_MODE_ENUM.eCAMT_RWMode, //Changeable CAM Table
                    0, //Master and Slave Start Mode. We will use bitwise: Bit 0 - Master Abs/Rel, Bit 1 - Slave Abs/Rel.
                    true, //Master position is Abs/Rel
                    true, //Slave position is Abs/Rel
                    out camTableHandle); //CAM Table Handler
                if (errVal != 0)
                {
                    ConnectData.ToOutput(string.Format("SelectTable return error {0}", errVal));
                }
                else
                {
                    ConnectData.ToOutput(string.Format("SelectTable - OK! CAM Table Handler = {0}", camTableHandle));
                }
            }
            catch (MMCException ex)
            {
                ConnectData.ToOutput(ex.What);
            }
        }

        private void buttonUnselectAll_Click(object sender, EventArgs e)
        {
            try
            {
                camTable.UnloadTable(0xFFFFFFFF);//Unselect All
                camTableHandle = 0xFFFFFFFF;
            }
            catch (MMCException ex)
            {
                ConnectData.ToOutput(ex.What);
            }
        }

        private void buttonCamIn_Click(object sender, EventArgs e)
        {
            try
            {
                Int16 errVal = ConnectData.SingleAxis.CamIn(
                    axisMaster.AxisReference, //The CAM Master Axis Ref
                    0, //Maser offset from master definition in CAM table
                    0, //Slave offset from slave definition in CAM table
                    1, //Maser scaling on master definition in CAM table 
                    1, //Slave scaling on master definition in CAM table
                    0, //Master Start Distance is backward distance from Master Sync Position to allow Ramp-In. 
                    //Untill we have Ram-In implemented it is always zero
                    0, //Master Sync Position defined as relative to first phase of master in CAM table. 
                    //If table is relative then it is defined as relative master position just like any other phase in the table
                    ECAM_VALUE_SRC_ENUM.eECAM_ACTUAL_VALUE, //Master Value Source
                    MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE, //Buffer Mode
                    0, //Overrides Start Mode of the SelectTable()
                    CURVE_TYPE_ENUM.eQuinticInterp, //CAM table Interpolation Type
                    ECAM_PERIODIC_ENUM.eCAM_NON_PERIODIC, //ECAM Periodic Mode is One shot, Periodic or Periodic-Linear
                    camTableHandle, //CAM Table Handler
                    true, //Auto Offset adjust slave position in table to axis position when master reaches ‘Sync Position’
                    true);//Table Access Handle
                if (errVal != 0)
                {
                    ConnectData.ToOutput(string.Format("CAM In return error {0}", errVal));
                }
                else
                {
                    ConnectData.ToOutput("CAM In - OK!");
                    ConnectData.ToOutput(string.Format("CAM In - OK! CAM Table Handler = {0}", camTableHandle));
                }
            }
            catch (MMCException ex)
            {
                ConnectData.ToOutput(ex.What);
            }
        }

        private void buttonCamOut_Click(object sender, EventArgs e)
        {
            try
            {
                Int16 errVal = ConnectData.SingleAxis.CamOut(true);
                if (errVal != 0)
                {
                    ConnectData.ToOutput(string.Format("CAM Out return error {0}", errVal));
                }
                else
                {
                    ConnectData.ToOutput("CAM Out - OK!");
                }
            }
            catch (MMCException ex)
            {
                ConnectData.ToOutput(ex.What);
            }
        }

        private void buttonCamStatus_Click(object sender, EventArgs e)
        {
            try
            {
                uint endOfProf;
                uint curIndex;
                uint cycle;
                Int16 errVal = ConnectData.SingleAxis.CamStatus(out endOfProf, out curIndex, out cycle);
                if (errVal != 0)
                {
                    ConnectData.ToOutput(string.Format("CAM Status return error {0}", errVal));
                }
                else
                {
                    ConnectData.ToOutput(string.Format(
                        "CAM Status: End of Profile {0}, Current Index {1}, Cycle {2}", endOfProf, curIndex, cycle));
                }
            }
            catch (MMCException ex)
            {
                ConnectData.ToOutput(ex.What);
            }
        }
    }
}
