﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;
using GMAS_Lib_test;

namespace MMCLibDotNET_Test_2010
{
    public partial class MMCErrorPolicyControl : UserControl, IParentPanel
    {
        #region Construct

        public MMCErrorPolicyControl()
        {
            InitializeComponent();
        }

        #endregion

        #region IParentPanel

        private Panel panel;
        private ConnectivityData connectData;

        public ConnectivityData ConnectData
        {
            get { return connectData; }
            set { connectData = value; }
        }

        public Panel GetPanel()
        {
            return panel;
        }

        public void Init(ConnectivityData data, Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += OnConnectDataChange;
            OnConnectDataChange();
        }

        private void OnConnectDataChange()
        {
            EnableButtons(ConnectData.Handle != 0 &&
                          ConnectData.SingleAxis != null);
        }

        private void EnableButtons(bool enable)
        {
            buttonResetSystemErrors.Enabled = enable;
            buttonGetPolicy.Enabled = enable;
            buttonRegisterPolicy.Enabled = enable;
        }

        #endregion

        #region Public Interface
        
        private void buttonResetSystemErrors_Click(object sender, EventArgs e)
        {
            try
            {
                MMCConnection.ResetSystemErrors(ConnectData.Handle);
            }
            catch (MMCException ex)
            {
                ConnectData.ToOutput(ex.What);
            }
        }

        private void buttonGetPolicy_Click(object sender, EventArgs e)
        {
            try
            {
                MC_ERROR_TYPE[] errorTypes = new[]
                    {
                        MC_ERROR_TYPE.eNODE_ERROR_ECAT_PHY_ERROR,
                        MC_ERROR_TYPE.eSYS_ERROR_CYCLIC_ERROR,
                        MC_ERROR_TYPE.eSYS_ERROR_MISSED_FRAMES,
                        MC_ERROR_TYPE.eNODE_ERROR_ECAT_AL_ERROR,
                        MC_ERROR_TYPE.eNODE_ERROR_UNEXPEC_MO_0,
                        MC_ERROR_TYPE.eNODE_ERROR_DRIVE_FAULT,
                        MC_ERROR_TYPE.eNODE_ERROR_QSTOP,
                        MC_ERROR_TYPE.eNODE_ERROR_HBT,
                        MC_ERROR_TYPE.eNODE_ERROR_EMCY,
                        MC_ERROR_TYPE.eNODE_ERROR_FB,
                    };
                NcGetPolicyEntry[] policies;
                MMCConnection.GetErrorPolicy(ConnectData.Handle, errorTypes, (byte) errorTypes.Length, ConnectData.SingleAxis.AxisReference, out policies);
                if (policies != null)
                {
                    ConnectData.ToOutput(string.Format("Policies for {0} :", ConnectData.SingleAxis.AxisName));
                    for (int i = 0; i < errorTypes.Length; i++)
                    {
                        ConnectData.ToOutput(string.Format("{0}) For {1} Policy: {2} , Threshold: {3} , Current Value: {4}", 
                            i + 1, errorTypes[i], policies[i].Policy, policies[i].Threshold, policies[i].CurrentVal));
                    }
                }
                else
                {
                    ConnectData.ToOutput("Can not to obtain policies...");
                }
            }
            catch (MMCException ex)
            {
                ConnectData.ToOutput(ex.What);
            }
        }

        private void buttonRegisterPolicy_Click(object sender, EventArgs e)
        {
            try
            {
                MC_ERROR_TYPE errType = MC_ERROR_TYPE.eNODE_ERROR_ECAT_PHY_ERROR;
                switch (comboBoxErType.Text)
                {
                    case "eNODE_ERROR_ECAT_PHY_ERROR":
                        errType = MC_ERROR_TYPE.eNODE_ERROR_ECAT_PHY_ERROR;
                        break;
                    case "eSYS_ERROR_CYCLIC_ERROR":
                        errType = MC_ERROR_TYPE.eSYS_ERROR_CYCLIC_ERROR;
                        break;
                    case "eSYS_ERROR_MISSED_FRAMES":
                        errType = MC_ERROR_TYPE.eSYS_ERROR_MISSED_FRAMES;
                        break;
                    case "eNODE_ERROR_ECAT_AL_ERROR":
                        errType = MC_ERROR_TYPE.eNODE_ERROR_ECAT_AL_ERROR;
                        break;
                    case "eNODE_ERROR_UNEXPEC_MO_0":
                        errType = MC_ERROR_TYPE.eNODE_ERROR_UNEXPEC_MO_0;
                        break;
                    case "eNODE_ERROR_DRIVE_FAULT":
                        errType = MC_ERROR_TYPE.eNODE_ERROR_DRIVE_FAULT;
                        break;
                    case "eNODE_ERROR_QSTOP":
                        errType = MC_ERROR_TYPE.eNODE_ERROR_QSTOP;
                        break;
                    case "eNODE_ERROR_HBT":
                        errType = MC_ERROR_TYPE.eNODE_ERROR_HBT;
                        break;
                    case "eNODE_ERROR_EMCY":
                        errType = MC_ERROR_TYPE.eNODE_ERROR_EMCY;
                        break;
                    case "eNODE_ERROR_FB":
                        errType = MC_ERROR_TYPE.eNODE_ERROR_FB;
                        break;
                }
                byte policy = Byte.Parse(textBoxErPolicy.Text);
                byte threshold = Byte.Parse(textBoxErThreshold.Text);
                NcPolicyEntry[] policies = new[]
                    {
                        new NcPolicyEntry(errType, policy, threshold),
                    };
                MMCConnection.RegisterErrorPolicy(ConnectData.Handle, 
                    policies, (byte) policies.Length, ConnectData.SingleAxis.AxisReference);
            }
            catch (MMCException ex)
            {
                ConnectData.ToOutput(string.Format("{0}; MMC Error {1}", ex.Message, ex.MMCError));
            }
        }

        #endregion
    }
}
