﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class MMCManagmentControl : UserControl, IParentPanel
    {
        public MMCManagmentControl()
        {
            InitializeComponent();
        }
        private System.Windows.Forms.Panel panel;
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }

        public Panel GetPanel()
        {
            return panel;
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
            ConnectData_OnDataChange();
        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.Handle == 0)
                EnableButtons(false);
            else
                EnableButtons(true);

        }

        private void EnableButtons(bool enable)
        {
            tbValues.Enabled = enable;
            tbRefCount.Enabled = enable;
            tbStartRef.Enabled = enable;
            btnHRread.Enabled = enable;
            btnHRwrite.Enabled = enable;
            btnChngOpMode.Enabled = enable;
            btnConfigPDO.Enabled = enable;
            btnReset.Enabled = enable;
            if (ConnectData != null && ConnectData.IsCANInternal)
                btnNetworkInfo.Enabled = enable;
            else
            {
                btnNetworkInfo.Enabled = false;
            }
        }

        private void OnLoad(object sender, EventArgs e)
        {
            Visible = true;
            Dock = DockStyle.Fill;
            Array arOPM = Enum.GetValues(typeof(OPM402));
            foreach (object obj in arOPM)
                cmbOpMode.Items.Add(obj);
            cmbOpMode.SelectedIndex = 0;

            Array arPDONum = Enum.GetValues(typeof(PDO_NUMBER_ENUM));
            foreach (object obj in arPDONum)
                cmbPDONum.Items.Add(obj);
            cmbPDONum.SelectedIndex = 0;

            Array arPDOType = Enum.GetValues(typeof(PDO_PARAM_TYPE_ENUM));
            foreach (object obj in arPDOType)
                cmbPDOType.Items.Add(obj);
            cmbPDOType.SelectedIndex = 0;

            Array boolParams = Enum.GetValues(typeof(MMC_BOOLEAN_PARAMETER_LIST_ENUM));
            foreach (object obj in boolParams)
                cmbBoolParams.Items.Add(obj);
            cmbBoolParams.SelectedIndex = 0;

            Array param = Enum.GetValues(typeof(MMC_PARAMETER_LIST_ENUM));
            foreach (object obj in param)
                cmbParams.Items.Add(obj);
            cmbParams.SelectedIndex = 0;
        }

        private void btnChngOpMode_Click(object sender, EventArgs e)
        {
            if (ConnectData.SingleAxis == null)
                return;
            OPM402 opMode = ConnectData.SingleAxis.GetOpMode();
            OPM402 mode = (OPM402)cmbOpMode.SelectedItem;
            if (opMode == mode) 
                return;
            try
            {
                ConnectData.SingleAxis.SetOpMode(mode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Operated Mode successfully changed to " + mode.ToString();
            ConnectData.ToOutput(res);
        }

        private void btnConfigPDO_Click(object sender, EventArgs e)
        {
            if (ConnectData.SingleAxis == null)
                return;
            PDO_NUMBER_ENUM pdoNum = (PDO_NUMBER_ENUM)cmbPDONum.SelectedItem;
            PDO_PARAM_TYPE_ENUM pdoType = (PDO_PARAM_TYPE_ENUM) cmbPDOType.SelectedItem;

            
            try
            {
                if( pdoType == PDO_PARAM_TYPE_ENUM.PDO_PARAM_REG)
                    ConnectData.SingleAxis.ConfigPDO(pdoNum, pdoType, 0, 0, 11, 254, 1, 1);
                else
                    ConnectData.SingleAxis.ConfigPDO(pdoNum, pdoType, 0, 0, (byte)NC_COMM_EVENT_GROUP.NC_COMM_EVENT_GROUP1, 255, 1, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "ConfigPDO command successfully Accepted. ";
            ConnectData.ToOutput(res);
        }

        private void btnReset_Click(object sender, EventArgs e)
        {
            if (ConnectData.SingleAxis != null)
            {
                try
                {
                    ConnectData.SingleAxis.Reset();
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    //return;
                }
            }
            if(ConnectData.GroupAxis != null)
            {
                byte sbAxcnt = 0;
                MMCSingleAxis[] axArr = ConnectData.GetGroupAxes(out sbAxcnt);
                foreach (MMCSingleAxis mmcSingleAxis in axArr)
                {
                    if (mmcSingleAxis != null)
                    {
                        try
                        {
                            mmcSingleAxis.Reset();
                        }
                        catch
                        {
                            continue;
                        }
                       
                    }
                }
                try
                {
                    ConnectData.GroupAxis.GroupReset();
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    //return;
                }
            }
            String res = "Reset command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnNetworkInfo_Click(object sender, EventArgs e)
        {
            
            MMCNetwork nt = new MMCNetwork( ConnectData.Handle);
            MmcNetworkInfo info = new MmcNetworkInfo();
            try
            {
                nt.GetNetworkInfo(ref info);
            }
            catch( MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "GetNetworkInfo command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void groupBox1_Enter(object sender, EventArgs e)
        {

        }

        private void btnHRread_Click(object sender, EventArgs e)
        {
            MMCHostComm hc = new MMCHostComm( ConnectData.Handle );
            short[] sar = new short[(int)MC_SIZES.MODBUS_IPC_RW_VALUES_COUNT];
            short err =0;
            ushort stat =0;
            int starRef = 0;
            int refCount = 2;
            starRef = Int32.Parse(tbStartRef.Text);
            refCount = Int32.Parse(tbRefCount.Text);
            try
            {
                hc.GMASReadHoldingRegisterTable(starRef, refCount, ref sar, ref err, ref stat);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "GMASReadHoldingRegisterTable command successfully completed.";
            res += "\n Accepted data [";
            for (int i = 0; i < refCount; i++)
                res += " " + sar[i].ToString() + " ";
            res += "]";
            ConnectData.ToOutput(res);

        }

        private void btnHRwrite_Click(object sender, EventArgs e)
        {
            MMCHostComm hc = new MMCHostComm(ConnectData.Handle);
            string str = tbValues.Text;
            string strVal = String.Empty;
            short[] sar = new short[(int)MC_SIZES.MODBUS_IPC_RW_VALUES_COUNT];
            int i = 0;
            while( str.IndexOf(',') > 0 )
            {
                strVal = str.Substring(0, str.IndexOf(','));
                sar[i] = Int16.Parse(strVal);
                i++;
                str = str.Substring(str.IndexOf(',') + 1);
            }
            sar[i] = Int16.Parse(str);
            int starRef = 0;
            int refCount = 2;
            starRef = Int32.Parse(tbStartRef.Text);
            refCount = Int32.Parse(tbRefCount.Text);
            try
            {
                hc.GMASWriteHoldingRegisterTable(starRef, refCount, sar);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            String res = "GMASWriteHoldingRegisterTable command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnReadBool_Click(object sender, EventArgs e)
        {
            UInt32 res = 0;
            MMC_BOOLEAN_PARAMETER_LIST_ENUM boolParam = (MMC_BOOLEAN_PARAMETER_LIST_ENUM)cmbBoolParams.SelectedItem;
            try
            {
                res = MMCConnection.ReadGlobalBoolParameter(ConnectData.Handle, boolParam, 0, 1);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            MessageBox.Show("Result of ReadGlobalBoolParameter = " + res.ToString() + "\nParam Name = " +
                            boolParam.ToString());
        }

        private void btnReadParam_Click(object sender, EventArgs e)
        {
            Double res = 0;
            MMC_PARAMETER_LIST_ENUM Param = (MMC_PARAMETER_LIST_ENUM)cmbParams.SelectedItem;
            try
            {
                res = MMCConnection.ReadGlobalParameter(ConnectData.Handle, Param, 0, 1);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            MessageBox.Show("Result of ReadGlobalParameter = " + res.ToString() + "\nParam Name = " +
                            Param.ToString());
        }

        private void btnSetDefaultGlobal_Click(object sender, EventArgs e)
        {
            try
            {
                MMCConnection.SetDefaultParametersGlobal(ConnectData.Handle);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "SetDefaultParamGlobal command successfully completed.";
            ConnectData.ToOutput(res);
            
        }
    }
}
