﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class MonitorControl : UserControl, IParentPanel
    {
        public MonitorControl()
        {
            InitializeComponent();
            Array ar = Enum.GetValues(typeof(MMC_BOOLEAN_PARAMETER_LIST_ENUM));
            foreach (object obj in ar)
                cmbReadBoolParam.Items.Add(obj);
            cmbReadBoolParam.SelectedIndex = 0;

            chbContMotion.Enabled = false; chbDis.Enabled = false; chbDiscretMotion.Enabled = false; chbError.Enabled = false;
            chbHome.Enabled = false; chbStandStill.Enabled = false; chbStop.Enabled = false; chbSyncMotion.Enabled = false;
            chbDecc.Enabled = false;chbAcc.Enabled = false;chbConstantVel.Enabled = false;
        }

        private void OnVisibleChange(object sender, EventArgs e)
        {

        }

        private void OnLoad(object sender, EventArgs e)
        {
            Visible = true;
            Dock = DockStyle.Fill;
        }

        private void btnPosAccept_Click(object sender, EventArgs e)
        {
            double tm = 0;
            double pos = -1;
            Stopper st = new Stopper();
            try
            {
                for (int i = 0; i < 100; i++)
                {
                    st.DebugStart();
                    pos = ConnectData.SingleAxis.GetActualPosition();//call function GetActualPosition();
                    st.DebugStop();
                    tm += st.Interval;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            String res = "Position = " + pos.ToString() +". Average time for GetActualPosition = " + (tm / 100).ToString() + " msec";
            ConnectData.ToOutput(res);
        }

        private void btnReadOpMode_Click(object sender, EventArgs e)
        {
            OPM402 mode = OPM402.OPM402_NO;

            try
            {
                mode = ConnectData.SingleAxis.GetOpMode();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Operation successfully accepted. Mode = " + mode.ToString() ;
            ConnectData.ToOutput(res);
        }

        private void btnReadAxisError_Click(object sender, EventArgs e)
        {
            ushort AxErr = 0;
            ushort emergCode = 0xFFFF;
            try
            {
                AxErr = ConnectData.SingleAxis.GetAxisError(ref emergCode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Axis Error successfully accepted. ErrorValue = " + AxErr.ToString() + ". EmergCode = " + emergCode.ToString();
            ConnectData.ToOutput(res);
        }

        private void btnReadStatus_Click(object sender, EventArgs e)
        {
            UInt32 status;
            try
            {
                status = ConnectData.SingleAxis.ReadStatus();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            
            uint plcStateMask = 0x7F8;//0x3F8;//0x7;//0x00000400
            uint validBitMask = 0x40000000;//0x3F8;//0x7;//0x00000400
            MC_STATE_SINGLE state = 0;
            if ((status & validBitMask) == validBitMask)
            {
                uint plc_state = status & plcStateMask;
                state = (MC_STATE_SINGLE)plc_state;
            }
            else
            {
                MessageBox.Show("Illegal state recognized .Possible Reason - GMAS is in Preoperational mode.",
                                "Monitor Control");
                ConnectData.Default();
                return;
            }

            chbContMotion.Checked = false; chbDis.Checked = false; chbDiscretMotion.Checked = false; chbError.Checked = false;
            chbHome.Checked = false; chbStandStill.Checked = false; chbStop.Checked = false; chbSyncMotion.Checked = false;
            chbDecc.Checked = false; chbAcc.Checked = false; chbConstantVel.Checked = false;
            if ((state & MC_STATE_SINGLE.ERROR_STOP) == MC_STATE_SINGLE.ERROR_STOP)
                chbError.Checked = true;
            if ((state & MC_STATE_SINGLE.DISABLED) == MC_STATE_SINGLE.DISABLED)
                chbDis.Checked = true;
            if ((state & MC_STATE_SINGLE.STOPPING) == MC_STATE_SINGLE.STOPPING)
                chbStop.Checked = true;
            if ((state & MC_STATE_SINGLE.STAND_STILL) == MC_STATE_SINGLE.STAND_STILL)
                chbStandStill.Checked = true;
            if ((state & MC_STATE_SINGLE.DISCRETE_MOTION) == MC_STATE_SINGLE.DISCRETE_MOTION)
                chbDiscretMotion.Checked = true;
            if ((state & MC_STATE_SINGLE.CONTINUOUS_MOTION) == MC_STATE_SINGLE.CONTINUOUS_MOTION)
                chbContMotion.Checked = true;
            if ((state & MC_STATE_SINGLE.SYNCHRONIZED_MOTION) == MC_STATE_SINGLE.SYNCHRONIZED_MOTION)
                chbSyncMotion.Checked = true;
            if ((state & MC_STATE_SINGLE.HOMING) == MC_STATE_SINGLE.HOMING)
                chbHome.Checked = true;
            if ((status & (UInt32)MC_STATE_SINGLE.ACCELERATING) == (UInt32)MC_STATE_SINGLE.ACCELERATING)
                chbAcc.Checked = true;
            if ((status & (UInt32)MC_STATE_SINGLE.DECELERATING) == (UInt32)MC_STATE_SINGLE.DECELERATING)
                chbDecc.Checked = true;
            if ((status & (UInt32)MC_STATE_SINGLE.CONSTANT_VELOCITY) == (UInt32)MC_STATE_SINGLE.CONSTANT_VELOCITY)
                chbConstantVel.Checked = true;
            
            String res = "Status of Axis " + ConnectData.SingleAxis.AxisName +
                            " successfully accepted. Status Value = 0x" + status.ToString("X") +
                              " (" + state.ToString() + ")";
            ConnectData.ToOutput(res);
        }

        private void btnReadBoolParam_Click(object sender, EventArgs e)
        {
            MMC_BOOLEAN_PARAMETER_LIST_ENUM param = (MMC_BOOLEAN_PARAMETER_LIST_ENUM)cmbReadBoolParam.SelectedItem;
            UInt32 Value;
            try
            {
                Value = ConnectData.SingleAxis.GetBoolParameter(param, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Value of Parameter " + param.ToString() + " successfully accepted. Value = " + Value.ToString();
            ConnectData.ToOutput(res);
        }

        private void btnActTorque_Click(object sender, EventArgs e)
        {
            double tm = 0;
            double pos = -1;
            Stopper st = new Stopper();
            try
            {
                for (int i = 0; i < 100; i++)
                {
                    st.DebugStart();
                    pos = ConnectData.SingleAxis.GetActualTorque();//call function GetActualPosition();
                    st.DebugStop();
                    tm += st.Interval;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            String res = "Torque = " + pos.ToString() + ". Average time for GetActualTorque = " + (tm / 100).ToString() + " msec";
            ConnectData.ToOutput(res);
        }

        private void btnActVelocity_Click(object sender, EventArgs e)
        {
            double tm = 0;
            double pos = -1;
            Stopper st = new Stopper();
            try
            {
                for (int i = 0; i < 100; i++)
                {
                    st.DebugStart();
                    pos = ConnectData.SingleAxis.GetActualVelocity();//call function GetActualPosition();
                    st.DebugStop();
                    tm += st.Interval;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            String res = "Velocity = " + pos.ToString() + ". Average time for GetActualVelocity = " + (tm / 100).ToString() + " msec";
            ConnectData.ToOutput(res);
        }

       
    }
}
