﻿using System;
using System.ComponentModel;
using System.Threading;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class MovingControl : UserControl, IParentPanel
    {
        public MovingControl()
        {
            InitializeComponent();
            Visible = false;
        }

        private System.Windows.Forms.Panel panel;
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }

        public System.Windows.Forms.Panel GetPanel()
        {
            return panel;
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
            ConnectData_OnDataChange();
        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.SingleAxis == null)
                EnableButtons(false);
            else
                EnableButtons(true);
            
        }

        private void EnableButtons( bool enable )
        {
            btnMoveAbsolute.Enabled = enable;
            
            btnPowerOn.Enabled = enable;
            btnPowerOff.Enabled = enable;
            btnStop.Enabled = enable;
            btnMoveRelative.Enabled = enable;
            btmMoveAdditive.Enabled = enable;
            btnSetMode.Enabled = enable;
            btnSetDefParam.Enabled = enable;
        }
        private void btnPowerOn_Click(object sender, EventArgs e)
        {
            
            try
            {
                //ConnectData.SingleAxis.SetHeartBeatConsumer(2);
                ConnectData.SingleAxis.PowerOn(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
                ConnectData.SingleAxis.EnableEndMotionEvent();
            }
            catch (MMCException ex)
            {
                String resolution, descript;
                resolution = descript = String.Empty;
                MessageBox.Show(ex.ToString(), "MMC Exception");
                
                try
                {
                    MMCConnection.GetErrorCodeDescriptionByID(ConnectData.Handle, (short)ex.MMCError, 0, out resolution, out descript);
                }
                catch
                {}

                try
                {
                    MMCConnection.GetErrorCodeDescriptionByID(ConnectData.Handle, (short)ex.MMCError, 1, out resolution, out descript);
                }
                catch
                { }

                try
                {
                    MMCConnection.GetErrorCodeDescriptionByID(ConnectData.Handle, (short)ex.MMCError, 2, out resolution, out descript);
                }
                catch
                { }

                try
                {
                    MMCConnection.GetErrorCodeDescriptionByID(ConnectData.Handle, (short)ex.MMCError, 3, out resolution, out descript);
                }
                catch
                { }
                

                MessageBox.Show(resolution, descript);
                
                return;
            }
            String res = "PowerOn command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnPowerOff_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.SingleAxis.PowerOff(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "PowerOff command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            if (!MotionPreview())
                return;

            try
            {
                ConnectData.SingleAxis.Stop(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Stop command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

        private void btnHome_Click(object sender, EventArgs e)
        {
            short err = 0;
            UInt32 FB_hndl = 0;
            try
            {
                ConnectData.SingleAxis.Home(ref err, ref FB_hndl);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Home command successfully Accepted. FB handle = " + FB_hndl.ToString();
            ConnectData.ToOutput(res);
        }

        private void btnMoveAbsolute_Click(object sender, EventArgs e)
        {
            if (!MotionPreview())
                return;
            int val = -1;
            double pos ;
            if(!Double.TryParse(tbDestinationPos.Text, out pos) )
            {
                MessageBox.Show("Invalid Position");
                return;
            }
            
            try
            {
                val = ConnectData.SingleAxis.MoveAbsolute(pos, ConnectData.SingleAxis.HomeBufferedMode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "MoveAbsolute command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

        private void btnMoveRelative_Click(object sender, EventArgs e)
        {
            if (!MotionPreview())
                return;

            int val = -1;
            double pos;
            if (!Double.TryParse(tbDistance.Text, out pos))
            {
                MessageBox.Show("Invalid Position");
                return;
            }

            try
            {
                val = ConnectData.SingleAxis.MoveRelative(pos, ConnectData.SingleAxis.HomeBufferedMode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "MoveRelative command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

        private void btmMoveAdditive_Click(object sender, EventArgs e)
        {
            if (!MotionPreview())
                return;

            int val = -1;
            double pos;
            if (!Double.TryParse(tbAdditiveDistance.Text, out pos))
            {
                MessageBox.Show("Invalid Position");
                return;
            }

            try
            {
                val = ConnectData.SingleAxis.MoveAdditive(pos, ConnectData.SingleAxis.HomeBufferedMode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "MoveAdditive command successfully Accepted.";
            ConnectData.ToOutput(res);

        }

        private void OnLoad(object sender, EventArgs e)
        {
            Visible = true;
            Dock = DockStyle.Fill;
        }

        private bool MotionPreview()
        {
            MC_STATE_SINGLE state = 0;
            uint plcStateMask = 0x7F8;//0x3F8;//0x7;//0x00000400
            uint validBitMask = 0x40000000;//0x3F8;//0x7;//0x00000400

            UInt32 rv = 0;

            try
            {
                rv = ConnectData.SingleAxis.ReadStatus();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                EnableButtons(false);
                MessageBox.Show("Please disconnect and reboot device", "Internal Communication Error");
                ConnectData.Default();
                return false;
            }

            if ((rv & validBitMask) == validBitMask)
            {
                state = (MC_STATE_SINGLE)(rv & plcStateMask);
            }

            if ((state & MC_STATE_SINGLE.ERROR_STOP) == MC_STATE_SINGLE.ERROR_STOP)
            {
                try
                {
                    ConnectData.SingleAxis.Reset();
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    EnableButtons(false);
                    MessageBox.Show("Please disconnect and reboot device", "Internal Communication Error");
                    ConnectData.Default();
                    return false;
                }
            }

            try
            {
                rv = ConnectData.SingleAxis.ReadStatus();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                EnableButtons(false);
                MessageBox.Show("Please disconnect and reboot device", "Internal Communication Error");
                ConnectData.Default();
                return false;
            }
            if ((rv & validBitMask) == validBitMask)
            {
                state = (MC_STATE_SINGLE)(rv & plcStateMask);
            }
            else
            {
                MessageBox.Show("Illegal state recognized .Possible Reason - GMAS is in Preoperational mode.",
                                "Homing Control");
                //ConnectData.Default();
                return false;
            }

            
            return true;
        }
        private void OnVisibleChange(object sender, EventArgs e)
        {
            if (Visible == false)
            {
                if (ConnectData != null && ConnectData.IsValid && ConnectData.SingleAxis != null)
                {
                    try
                    {
                        ConnectData.SingleAxis.PowerOff(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
                    }
                    catch (MMCException ex)
                    {
                        MessageBox.Show(ex.ToString(), "MMC Exception");
                        return;
                    }
                }
            }
            else
            {
                if (ConnectData != null && ConnectData.IsValid && ConnectData.SingleAxis != null)
                {
                    //MotionPreview();
                }
            }
        }

        private void btnSetMode_Click(object sender, EventArgs e)
        {
            OPM402 opMode = ConnectData.SingleAxis.GetOpMode();
            switch (ConnectData.IsCANInternal)
            {
                case true:
                    if (opMode != OPM402.OPM402_INTERPOLATED_POSITION_MODE)
                    {
                        try
                        {
                            ConnectData.SingleAxis.SetOpMode(OPM402.OPM402_INTERPOLATED_POSITION_MODE);
                        }
                        catch (MMCException ex)
                        {
                            MessageBox.Show(ex.ToString(), "Moving Control");
                        }
                        String res = "Operated Mode successfully changed to " + OPM402.OPM402_INTERPOLATED_POSITION_MODE.ToString();
                        ConnectData.ToOutput(res);
                        return;
                    }
                    break;
                case false:
                    if (opMode != OPM402.OPM402_CYCLIC_SYNC_POSITION_MODE)
                    {
                        try
                        {
                            ConnectData.SingleAxis.SetOpMode(OPM402.OPM402_CYCLIC_SYNC_POSITION_MODE);
                        }
                        catch (MMCException ex)
                        {
                            MessageBox.Show(ex.ToString(), "Moving Control");
                        }
                        String res = "Operated Mode successfully changed to " + OPM402.OPM402_CYCLIC_SYNC_POSITION_MODE.ToString();
                        ConnectData.ToOutput(res);
                        return;
                    }
                    break;
            }
        }

        private void btnSetDefParam_Click(object sender, EventArgs e)
        {
            MMC_MOTIONPARAMS_SINGLE singleParam = new MMC_MOTIONPARAMS_SINGLE();
            singleParam.fAcceleration = 1000;
            singleParam.fDeceleration = 1000;
            singleParam.fVelocity = 1000;
            singleParam.eDirection = MC_DIRECTION_ENUM.MC_POSITIVE_DIRECTION;
            singleParam.fJerk = 1000000;
            singleParam.eBufferMode = MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE;
            singleParam.ucExecute = 1;
            ConnectData.SingleAxis.SetDefaultParams(singleParam);
        }

        private void btnMoveAdditiveRepetitive_Click(object sender, EventArgs e)
        {
            if (!MotionPreview())
                return;

            int val = -1;
            double pos;
            if (!Double.TryParse(tbMoveRepetitive.Text, out pos))
            {
                MessageBox.Show("Invalid Position");
                return;
            }

            try
            {
                val = ConnectData.SingleAxis.MoveAdditiveRepetitive(pos, MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "MoveAdditiveRepetitive command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

        private void btnMoveAbsoluteRepetitive_Click(object sender, EventArgs e)
        {
            if (!MotionPreview())
                return;

            int val = -1;
            double pos;
            if (!Double.TryParse(tbMoveRepetitive.Text, out pos))
            {
                MessageBox.Show("Invalid Position");
                return;
            }

            try
            {
                val = ConnectData.SingleAxis.MoveAbsoluteRepetitive(pos, MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "MoveAbsoluteRepetitive command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

        private delegate void ToOutputDelegate(Object obj);
        private void ToOutput(Object obj)
        {
            ConnectData.ToOutput(obj);
        }
        private void SRStatuses()
        {
            BackgroundWorker _bw = new BackgroundWorker();
            _bw.DoWork += new DoWorkEventHandler(_bw_DoWork);
            _bw.RunWorkerAsync();
        }

        void _bw_DoWork(object sender, DoWorkEventArgs e)
        {
            Int32 val = -1;
            byte[] rq = new byte[2];
            rq[0] = 0x53;
            rq[1] = 0x52;
            while (ConnectData.Handle != 0 && ConnectData.SingleAxis != null)
            {
                try
                {
                    ConnectData.SingleAxis.ElmoGetSyncParam(rq, ref val);
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    return;
                }
                String res = "Elmo GetSyncInteger Param command successfully Accepted. Param " + "SR" +
                             " = " + val.ToString();
                if (InvokeRequired)
                {
                    object[] objArr = new object[1];
                    objArr[0] = res;
                    Invoke(new ToOutputDelegate(ToOutput), objArr);
                }
                else
                    ConnectData.ToOutput(res);
            }
        }
        private void btnMoveRelativePepetative_Click(object sender, EventArgs e)
        {
            if (!MotionPreview())
                return;

            int val = -1;
            double pos;
            if (!Double.TryParse(tbMoveRepetitive.Text, out pos))
            {
                MessageBox.Show("Invalid Position");
                return;
            }

            try
            {
                val = ConnectData.SingleAxis.MoveRelativeRepetitive(pos, MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "MoveRelativeRepetitive command successfully Accepted.";
            if (InvokeRequired)
            {
                object[] objArr = new object[1];
                objArr[0] = res;
                Invoke(new ToOutputDelegate(ToOutput), objArr);
            }
            else
                ConnectData.ToOutput(res);
            SRStatuses();
        }

        private void eomTHread()
        {
            bool bl = true;
            while (ConnectData != null && ConnectData.SingleAxis != null)
            {
                try
                {
                    if (bl == true)
                    {
                        ConnectData.SingleAxis.MoveAbsolute(0, 50000, 1000000, 1000000, 10000000,
                                                            MC_DIRECTION_ENUM.MC_POSITIVE_DIRECTION,
                                                            MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
                        bl = false;
                    }
                    else
                    {
                        ConnectData.SingleAxis.MoveAbsolute(10000, 50000, 2000000, 2000000, 10000000,
                                                            MC_DIRECTION_ENUM.MC_POSITIVE_DIRECTION,
                                                            MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
                        bl = true;
                    }
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    //return;
                }
                String res = "MoveAbsolute command successfully Accepted.";
                
                if (InvokeRequired)
                {
                    object[] objArr = new object[1];
                    objArr[0] = res;
                    Invoke(new ToOutputDelegate(ToOutput), objArr);
                }
                else
                    ConnectData.ToOutput(res);
                System.Threading.Thread.Sleep(2000);
               
            }

        }

        private void btnEOM_Click(object sender, EventArgs e)
        {
            Thread eom = new Thread(eomTHread);
            
            eom.Start();
        }

        
    }
}
