﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;
using GMAS_Lib_test;

namespace MMCLibDotNET_Test_2010
{
    public partial class PvtControl : UserControl, IParentPanel
    {
        public PvtControl()
        {
            InitializeComponent();
        }

        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }

        private Panel panel;
        public Panel GetPanel()
        {
            return panel;
        }

        public void Init(ConnectivityData data, Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += ConnectData_OnDataChange;
            ConnectData_OnDataChange();
        }

        void ConnectData_OnDataChange()
        {
            EnableButtons(ConnectData.Handle != 0);
        }

        private void EnableButtons(bool val)
        {
            buttonMotorOn.Enabled = val && ConnectData.SingleAxis != null;
            buttonMotorOff.Enabled = val && ConnectData.SingleAxis != null;
            buttonLoadTable.Enabled = val && ConnectData.SingleAxis != null;
            buttonInitTable.Enabled = val && ConnectData.SingleAxis != null;
            buttonMoveTable.Enabled = val && ConnectData.SingleAxis != null;
            buttonUnselectAll.Enabled = val;
        }

        private uint tableHandle;
        protected uint TableHandle
        {
            get { return tableHandle; }
            set { tableHandle = value; }
        }

        private void buttonLoadTable_Click(object sender, EventArgs e)
        {
            string path = textBoxTable.Text;
            if (string.IsNullOrEmpty(path))
            {
                MessageBox.Show("Invalid table path");
                return;
            }
            float[] transitionParams = new float[1] { 0 };
            try
            {
                TableHandle = MMCConnection.LoadTableFromFile(ConnectData.Handle, transitionParams, 
                                                              NC_TRANSITION_MODE_ENUM.MC_TM_NONE_MODE,
                                                              MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE,
                                                              NC_MOTION_TABLE_TYPE_ENUM.eNC_TABLE_PVT_FILE,
                                                              MC_COORD_SYSTEM_ENUM.MC_NONE_COORD,
                                                              ConnectData.SingleAxis.AxisReference, path);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(string.Format("{0}. Error {1}", ex.What, ex.MMCError));
            }
        }

        private void buttonInitTable_Click(object sender, EventArgs e)
        {
            try
            {
                const int pvtTabRowsNum = 360;
                const int dataInTable = pvtTabRowsNum*3;
                double[] pvtTab = new double[dataInTable];
                for (int i = 0; i < dataInTable; i = i + 3)
                {
                    pvtTab[i] = 0.5;//seconds
                    pvtTab[i + 1] = 10000.0 * Math.Cos(Math.PI * 2.0 * (i - 1.0) / 360.0);
                    pvtTab[i + 2] = 10000.0 * Math.Sin(Math.PI * 2.0 * (-1) * (i - 1.0) / 360.0) / 0.02;
                }
                float[] transitionParams = new float[1] { 0 };
                TableHandle = MMCConnection.InitTable(ConnectData.Handle, transitionParams,
                                                      NC_TRANSITION_MODE_ENUM.MC_TM_NONE_MODE,
                                                      MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE,
                                                      NC_MOTION_TABLE_TYPE_ENUM.eNC_TABLE_PVT_ARRAY,
                                                      MC_COORD_SYSTEM_ENUM.MC_NONE_COORD,
                                                      pvtTabRowsNum, 60, ConnectData.SingleAxis.AxisReference,
                                                      1, 1, 1, 0, 0, 1);
                int pos = 0;
                byte bAuto = 0;
                const int pvtTabRowsInPortion = 40;
                const int dataInPortion = pvtTabRowsInPortion*3;
                for (int i = 0; i < 9; i++)
                {
                    double[] tempArr = new double[dataInPortion];
                    Array.Copy(pvtTab, pos, tempArr, 0, dataInPortion);
                    MMCConnection.AppendPointsToTable(ConnectData.Handle, tempArr,
                                                      NC_MOTION_TABLE_TYPE_ENUM.eNC_TABLE_PVT_ARRAY,
                                                      TableHandle, 0, pvtTabRowsInPortion, 0, bAuto);
                    pos += dataInPortion;
                    bAuto = 1;
                }
            }
            catch (MMCException ex)
            {
                MessageBox.Show(string.Format("{0}. Error {1}", ex.What, ex.MMCError));
            }
        }

        private void buttonUnselectAll_Click(object sender, EventArgs e)
        {
            try
            {
                MMCConnection.UnloadTable(ConnectData.Handle, 0xFFFFFFFF);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(string.Format("{0}. Error {1}", ex.What, ex.MMCError));
            }
        }

        private void buttonMoveTable_Click(object sender, EventArgs e)
        {
            try
            {
                uint retVal = ConnectData.SingleAxis.MoveTable(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE, TableHandle, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(string.Format("{0}. Error {1}", ex.What, ex.MMCError));
            }
        }

        private void buttonMotorOn_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.SingleAxis.PowerOn(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
                while ((ConnectData.SingleAxis.ReadStatus() & (uint) MC_STATE_SINGLE.STAND_STILL) !=
                       (uint) MC_STATE_SINGLE.STAND_STILL)
                {
                }                           
            }
            catch (MMCException ex)
            {
                MessageBox.Show(string.Format("{0}. Error {1}", ex.What, ex.MMCError));
            }
        }

        private void buttonMotorOff_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.SingleAxis.PowerOff(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
                while ((ConnectData.SingleAxis.ReadStatus() & (uint) MC_STATE_SINGLE.DISABLED) !=
                       (uint) MC_STATE_SINGLE.DISABLED)
                {
                }
            }
            catch (MMCException ex)
            {
                MessageBox.Show(string.Format("{0}. Error {1}", ex.What, ex.MMCError));
            }
        }
    }
}
