using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using GMAS_Lib_test;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace MMCLibDotNET_Test_2010
{
    public partial class ResDownload : UserControl, IParentPanel
    {
        public ResDownload()
        {
            InitializeComponent();
        }
        private Panel mainPanel;
        public Panel GetPanel()
        {
            return mainPanel;
        }

        private ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                if (connectData != null)
                {
                    connectData.OnDataChange -= connectData_OnDataChange;
                }

                connectData = value;
                
                if (connectData != null)
                {
                    connectData.OnDataChange += connectData_OnDataChange; 
                }
            }
        }

        void connectData_OnDataChange()
        {
            if (connectData != null)
            {
                buttonDownRes.Enabled = connectData.Handle != 0;
                
                if (connectData.IsCANInternal)
                {
                    textBoxPathCfg.Enabled = false;
                    buttonBrowseCfg.Enabled = false;
                    buttonDownCfg.Enabled = false;
                }
                else
                {
                    textBoxPathCfg.Enabled = true;
                    buttonBrowseCfg.Enabled = true;
                    buttonDownCfg.Enabled = connectData.Handle != 0;
                }
            }
        }

        public void Init(ConnectivityData data, Panel pnl)
        {
            ConnectData = data;
            mainPanel = pnl;
            connectData_OnDataChange();//for GUI initialization
        }
 #region Download Resources
        private void buttonBrowseRes_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "G-MAS Resource file (MMCResources.xml)|MMCResources.xml";
            if (dlg.ShowDialog() != DialogResult.OK)
                return;
            textBoxPathRes.Text = dlg.FileName;

        }
        private void buttonDownRes_Click(object sender, EventArgs e)
        {
            if (System.IO.File.Exists(textBoxPathRes.Text) == false)
            {
                MessageBox.Show("Invalid path " + textBoxPathRes.Text, "Resource Download");
                return;
            }
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "Resource Download");
                return;
            }
            if (MMCConnection.DownloadResouceFile(ConnectData.Handle,
                textBoxPathRes.Text, OnStartRes, OnProgressRes, OnFinishRes) == false)
            {
                MessageBox.Show("Download Resouce File can not be started", "Resources Download");
                return;
            }
        }

        private void OnStartRes(int handle, LoadType type)
        {
            ConnectData.ToOutput("Resource Download started!");
            iterator = 0;
        }

        private void OnProgressRes(int comHandle, LoadType type, uint dataSize, Stage stage)
        {
            ConnectData.ToOutput(string.Format("Resource Download: stage - {0} {1}", stage, dataSize));
            Application.DoEvents();
        }

        int iterator;
        private void OnFinishRes(int comHandle, LoadType type, Error error)
        {
            //if (iterator >= model.TargetsData.Count)
            //    return;

            if (error != Error.OK)
            {
                ConnectData.ToOutput(string.Format("Resource Download Failed: {0}", error));
                return;
            }
            ConnectData.ToOutput(string.Format("Resource Download OK: {0}", iterator));

            //WaitGmasToConnect(trg);
            iterator++;
        }

 #endregion
 #region Download Config
        private void buttonBrowseCfg_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "G-MAS ECAT configuration file (Cfg.xml)|Cfg.xml";
            if (dlg.ShowDialog() != DialogResult.OK)
                return;
            textBoxPathCfg.Text = dlg.FileName;

        }

        private void buttonDownCfg_Click(object sender, EventArgs e)
        {
            if (System.IO.File.Exists(textBoxPathCfg.Text) == false)
            {
                MessageBox.Show("Invalid path " + textBoxPathCfg.Text, "Config Download");
                return;
            }
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "Config Download");
                return;
            }


            if (MMCConnection.DownloadECATConfigFile(ConnectData.Handle,
                 textBoxPathCfg.Text, OnStartCfg, OnProgressCfg, OnFinishCfg) == false)
            {
                  MessageBox.Show("Download ECAT ConfigFile can not be started", "Config Download");
                  return;
            }
        }
        private void OnStartCfg(int handle, LoadType type)
        {
            ConnectData.ToOutput("Config Download started!");
            iterator = 0;
        }

        private void OnProgressCfg(int comHandle, LoadType type, uint dataSize, Stage stage)
        {
            ConnectData.ToOutput(string.Format("ECAT ConfigFile Download: stage - {0} {1}", stage, dataSize));
            Application.DoEvents();
        }

    
        private void OnFinishCfg(int comHandle, LoadType type, Error error)
        {
            //if (iterator >= model.TargetsData.Count)
            //    return;

            if (error != Error.OK)
            {
                ConnectData.ToOutput(string.Format("Config Download Failed: {0}", error));
                return;
            }
            ConnectData.ToOutput(string.Format("Config Download OK: {0}", iterator));

            //WaitGmasToConnect(trg);
            iterator++;
        }

 #endregion

        private void buttonRestart_Click(object sender, EventArgs e)
        {
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "Restart G-MAS");
                return;
            }

            ConnectData.ToOutput("Restart G-MAS");
            MMCConnection.ResetControl(ConnectData.Handle);

            Thread.Sleep(1000);

            MMCConnection.CloseConnection(ConnectData.Handle);

            ConnectData.Handle = 0;

        }
    }
}
