﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class SingleAxisIOControl : UserControl, IParentPanel
    {
        public SingleAxisIOControl()
        {
            InitializeComponent();
        }

        private System.Windows.Forms.Panel panel;
        public Panel GetPanel()
        {
            return panel;
        }
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
           
            ConnectData_OnDataChange();
        }
        void ConnectData_OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.SingleAxis == null)
                EnableButtons(false);
            else
            {
                EnableButtons(true);
                
            }

        }
        private void EnableButtons(bool enable)
        {
            if (enable == true)
            {
                label1.Text = "I/O Control For Axis " + ConnectData.SingleAxis.AxisName;
            }
            else
            {
                label1.Text = "I/O Control For Axis ";
            }
            btnGetDigInputs.Enabled = enable;
            btnGetDigInput.Enabled = enable;
            btnGetDigOutput.Enabled = enable;
            btnConfigVirtualEncoder.Enabled = enable;
            btnCancelVirtualEncoder.Enabled = enable;

            tbInputNum.Enabled = enable;
            tbOutputNum.Enabled = enable;
            
        }

        private void btnGetDigInput_Click(object sender, EventArgs e)
        {
            byte rs = 0;
            Int32 val = -1;// = 10;
            if (!Int32.TryParse(tbInputNum.Text, out val))
            {
                MessageBox.Show("Invalid Value");
                return;
            }
            if (val < 0)
            {
                MessageBox.Show("Negative Index");
                return;
            }
            try
            {
                rs = ConnectData.SingleAxis.GetDigInput(val);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "GetDigitalInput command successfully completed. Accepted Value = " + rs.ToString();
            ConnectData.ToOutput(res);
        }

        private void btnGetDigInputs_Click(object sender, EventArgs e)
        {
            UInt32 rs = 0;
            try
            {
                rs = ConnectData.SingleAxis.GetDigInputs();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "GetDigitalInputs command successfully completed. Accepted Value = " + rs.ToString("X");
            ConnectData.ToOutput(res);
        }

        private void btnGetDigOutput_Click(object sender, EventArgs e)
        {
            byte rs = 0;
            Int32 val = -1;// = 10;
            if (!Int32.TryParse(tbOutputNum.Text, out val))
            {
                MessageBox.Show("Invalid Value");
                return;
            }
            if (val < 0)
            {
                MessageBox.Show("Negative Index");
                return;
            }
            try
            {
                rs = ConnectData.SingleAxis.GetDigOutputs(val);
                uint iRes;
                iRes = ConnectData.SingleAxis.GetDigOutputs32bit(val);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "GetDigOutputs command successfully completed. Accepted Value = " + rs.ToString();
            ConnectData.ToOutput(res);

        }

        private void btnConfigVirtualEncoder_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.SingleAxis.ConfigVirtualEncoder( -1000000, 1000000, 1, 0, 100);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "ConfigVirtualEncoder command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnCancelVirtualEncoder_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.SingleAxis.CancelVirtualEncoder();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "CancelVirtualEncoder command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnSetDigOutput_Click(object sender, EventArgs e)
        {
            
            try
            {
                //ConnectData.SingleAxis.SetDigOutputs(1,16,1);
                ConnectData.SingleAxis.SetDigOutputs32Bit(1,16);

                ConnectData.SingleAxis.SetDigOutputs(1, 0, 1);
                //ConnectData.SingleAxis.SetDigOutputs32Bit(1, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
        }

        private void btnPIVarInfo_Click(object sender, EventArgs e)
        {
            NC_PI_ENTRY entry = new NC_PI_ENTRY();
            ConnectData.SingleAxis.GetPIVarInfo(0, PIVarDirection.ePI_OUTPUT, ref entry);
            PI_VAR_UNION varUnion = new PI_VAR_UNION();
            ConnectData.SingleAxis.ReadPIVar(0, PIVarDirection.ePI_OUTPUT, VAR_TYPE.INT, ref varUnion);
            varUnion._uint32 = 10;
            ConnectData.SingleAxis.WritePIVar(0, varUnion, VAR_TYPE.INT);

            PI_VAR_UNION r_varUnion = new PI_VAR_UNION();
            ConnectData.SingleAxis.ReadPIVar(0, PIVarDirection.ePI_OUTPUT, VAR_TYPE.INT, ref r_varUnion);
        }

        private void btnRW_Memory_Click(object sender, EventArgs e)
        {
            byte[] result;
            ConnectData.SingleAxis.EthercatReadMemoryRange( 0, 8, out result);

            for (int i = 0; i < 8; i++)
                result[i] = (byte)i;
            byte[] wArr = new byte[8];
            for (int i = 0; i < 8; i++)
                wArr[i] = (byte)i;

            try
            {
                ConnectData.SingleAxis.EthercatWriteMemoryRange(0, 8, wArr);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
            }

            try
            {
                ConnectData.SingleAxis.EthercatReadMemoryRange(0, 8, out result);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
            }
           
        }

        private void btnRW_LargeVars_Click(object sender, EventArgs e)
        {
            PI_LARGE_VAR_UNION varUnion = new PI_LARGE_VAR_UNION();
            ConnectData.SingleAxis.ReadLargePIVar(0, PIVarDirection.ePI_OUTPUT, LARGE_VAR_TYPE.DOUBLE, ref varUnion);
        }

    }
}
