﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class SingleAxisManagmentControl : UserControl, IParentPanel
    {
        public SingleAxisManagmentControl()
        {
            InitializeComponent();
        }

        private System.Windows.Forms.Panel panel;
        public Panel GetPanel()
        {
            return panel;
        }
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
            Array arDir = Enum.GetValues(typeof(MC_DIRECTION_ENUM));
            foreach (object obj in arDir)
                cmbDir.Items.Add(obj);

            Array arOPM = Enum.GetValues(typeof(MC_BUFFERED_MODE_ENUM));
            foreach (object obj in arOPM)
                cmbBuffMode.Items.Add(obj);
            ConnectData_OnDataChange();
        }
        void ConnectData_OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.SingleAxis == null)
                EnableButtons(false);
            else
            {
                EnableButtons(true);
                tbAcc.Text = ConnectData.SingleAxis.Acceleration.ToString();
                tbDec.Text = ConnectData.SingleAxis.Deceleration.ToString();
                tbJerk.Text = ConnectData.SingleAxis.Jerk.ToString();
                tbVelocity.Text = ConnectData.SingleAxis.Velocity.ToString();


                cmbBuffMode.SelectedItem = ConnectData.SingleAxis.HomeBufferedMode;

                cmbDir.SelectedItem = ConnectData.SingleAxis.Direction;
            }

        }
        private void EnableButtons(bool enable)
        {
            if(enable == true)
            {
                label1.Text = "For Axis " + ConnectData.SingleAxis.AxisName;
            }
            else
            {
                label1.Text = String.Empty;
            }
            btnMotionEventON.Enabled = enable;
            btnMotionEventOFF.Enabled = enable;

            tbAcc.Enabled = enable;
            tbDec.Enabled = enable;
            tbJerk.Enabled = enable;
            tbVelocity.Enabled = enable;
            cmbDir.Enabled = enable;
            btnSet.Enabled = enable;
            cmbBuffMode.Enabled = enable;
            btnGenPDORead.Enabled = enable;
            btnUploadSDO.Enabled = enable;
            btnGenPDOWrite.Enabled = enable;
            btnSDODown.Enabled = enable;
            btnSetDefParam.Enabled = enable;
        }

        private void OnLoad(object sender, EventArgs e)
        {
            Visible = true;
            Dock = DockStyle.Fill;
        }

        private void btnMotionEventON_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.SingleAxis.EnableEndMotionEvent();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "EnableEndMotionEvent command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnMotionEventOFF_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.SingleAxis.DisableEndMotionEvent();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "DisableEndMotionEvent command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnSet_Click(object sender, EventArgs e)
        {
            if (!Double.TryParse(tbAcc.Text, out ConnectData.SingleAxis.Acceleration))
                MessageBox.Show("Acceleration Value Error", "Argument Error");

            if (!Double.TryParse(tbDec.Text, out ConnectData.SingleAxis.Deceleration))
                MessageBox.Show("Deceleration Value Error", "Argument Error");

            if (!Double.TryParse(tbJerk.Text, out ConnectData.SingleAxis.Jerk))
                MessageBox.Show("Jerk Value Error", "Argument Error");

            if (!Double.TryParse(tbVelocity.Text, out ConnectData.SingleAxis.Velocity))
                MessageBox.Show("Velocity Value Error", "Argument Error");

            ConnectData.SingleAxis.Direction = (MC_DIRECTION_ENUM)cmbDir.SelectedItem;
            ConnectData.SingleAxis.HomeBufferedMode = (MC_BUFFERED_MODE_ENUM)cmbBuffMode.SelectedItem;
        }

        private void btnGenPDORead_Click(object sender, EventArgs e)
        {
            UInt64 res = 0;
            try
            {
                res = ConnectData.SingleAxis.PDOGeneralRead(0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String str = "PDOGeneralRead command successfully completed. Accepted value = " + res.ToString();
            ConnectData.ToOutput(str);
        }

        private void btnGenPDOWrite_Click(object sender, EventArgs e)
        {
            
            try
            {
                ConnectData.SingleAxis.PDOGeneralWrite(10, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String str = "PDOGeneralRead command successfully completed.";
            ConnectData.ToOutput(str);
        }
        private void btnSDODown_Click(object sender, EventArgs e)
        {
            //SendSdoDownloadCmd(Int32 lData, byte ucSubIndex, UInt32 ulDataLength, ushort usIndex, ushort usSlaveID)
            //ConnectData.SingleAxis.SendSdoDownloadCmd(0,0,1,0x22a0,0);
            try
            {
                ConnectData.SingleAxis.DownloadSDO(0x22A0, 0, (Byte) 0, 1000);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String str = "SendSdoDownloadCmd command successfully completed. Accepted value = " ;
            ConnectData.ToOutput(str);
        }

        private void btnUploadSDO_Click(object sender, EventArgs e)
        {
            int iRes;
           
            try
            {
                //public Int32 SendSdoUploadCmd( byte ucSubIndex, UInt32 ulDataLength, ushort usIndex, ushort usSlaveID) //throw (CMMCException)
                //iRes = ConnectData.SingleAxis.SendSdoUploadCmd(0, 4, 0x6401, 2);   
                //iRes = ConnectData.SingleAxis.SendSdoUploadCmd( 0, 2, 0x2f05, 0);
                ConnectData.SingleAxis.UploadSDO(0x2f05, 0, out iRes, 5000);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String str = "SendSdoUploadCmd command successfully completed. Accepted value = " + iRes.ToString();
            ConnectData.ToOutput(str);
        }

        private void btnSetDefParam_Click(object sender, EventArgs e)
        {
            if (ConnectData.SingleAxis == null)
                return;
            try
            {
                ConnectData.SingleAxis.SetDefaultParameters();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String str = "SetDefaultManufacturerParameters command successfully completed.";
            ConnectData.ToOutput(str);
        }

        
    }
}
