﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class SingleAxisPI_VarControl : UserControl, IParentPanel
    {
        public SingleAxisPI_VarControl()
        {
            InitializeComponent();
        }

        private System.Windows.Forms.Panel panel;
        public Panel GetPanel()
        {
            return panel;
        }
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);

            ConnectData_OnDataChange();

            Array ar = Enum.GetValues(typeof(PIVarDirection));
            foreach (Object obj in ar)
            {
                cmbDirection.Items.Add(obj);
            }
            cmbDirection.SelectedIndex = 0;

            ar = Enum.GetValues(typeof(VAR_TYPE));
            foreach (Object obj in ar)
            {
                cmbVarType.Items.Add(obj);
            }
            cmbVarType.SelectedIndex = 0;

            ar = Enum.GetValues(typeof(LARGE_VAR_TYPE));
            foreach (Object obj in ar)
            {
                cmbLargeVarType.Items.Add(obj);
            }
            cmbLargeVarType.SelectedIndex = 0;



        }
        void ConnectData_OnDataChange()
        {
            //if (ConnectData.IsValid == false || ConnectData.SingleAxis == null)
            //    EnableButtons(false);
            //else
            //{
            //    EnableButtons(true);

            //}

        }

        private void btnReadVarInfo_Click(object sender, EventArgs e)
        {
            string str = tbIndex.Text;
            UInt16 index;
            if (!UInt16.TryParse(str, out index))
            {
                MessageBox.Show("Index reading fail");
                return;
            }
            PIVarDirection dir = (PIVarDirection)cmbDirection.SelectedItem;
            NC_PI_ENTRY entry = new NC_PI_ENTRY();
            try
            {
                ConnectData.SingleAxis.GetPIVarInfo(index, dir, ref entry);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            char[] ch = new char[entry.pAliasing.Length];
            for (int i = 0; i < entry.pAliasing.Length; i++)
            {
                ch[i] = (char)entry.pAliasing[i];
            }
            str = new string(ch);
            ConnectData.ToOutput("Alias = " + str);
            ConnectData.ToOutput( "Var type = "+ entry.ucVarType );
            ConnectData.ToOutput("Bit Size = " + entry.uiBitSize);
            ConnectData.ToOutput("Bit Offset = " + entry.uiBitOffset);

        }

        private void btnRead_var_Click(object sender, EventArgs e)
        {
            string str = tbIndex.Text;
            UInt16 index;
            if (!UInt16.TryParse(str, out index))
            {
                MessageBox.Show("Index reading fail");
                return;
            }
            PIVarDirection dir = (PIVarDirection)cmbDirection.SelectedItem;
            VAR_TYPE varT = (VAR_TYPE)cmbVarType.SelectedItem;
            PI_VAR_UNION varUnion = new PI_VAR_UNION();
            try
            {
                ConnectData.SingleAxis.ReadPIVar(index, dir, varT, ref varUnion);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            
            switch (varT)
            {
                   case VAR_TYPE.BYTE:
                    tbVarValue.Text = varUnion._byte.ToString();
                    break;
                    case VAR_TYPE.S_BYTE:
                    tbVarValue.Text = varUnion.s_byte.ToString();
                    break;
                    case VAR_TYPE.SHORT:
                    tbVarValue.Text = varUnion._int16.ToString();
                    break;
                    case VAR_TYPE.USHORT:
                    tbVarValue.Text = varUnion._uint16.ToString();
                    break;
                    case VAR_TYPE.INT:
                    tbVarValue.Text = varUnion._int32.ToString();
                    break;
                    case VAR_TYPE.UINT:
                    tbVarValue.Text = varUnion._uint32.ToString();
                    break;
                    case VAR_TYPE.FLOAT:
                    tbVarValue.Text = varUnion._single.ToString();
                    break;
                    case VAR_TYPE.BYTE_ARR:
                    string outString = "Read Byte Array completed. Results= '";
                    for (int i = 0; i < varUnion._rawData.Length; i++)
                    {
                        outString += varUnion._rawData[i].ToString() + "'";
                    }
                    ConnectData.ToOutput( outString);
                    break;
            }
        }

        private void btnWriteVar_Click(object sender, EventArgs e)
        {
            string str = tbIndex.Text;
            UInt16 index;
            if (!UInt16.TryParse(str, out index))
            {
                MessageBox.Show("Index reading fail");
                return;
            }
            
            VAR_TYPE varT = (VAR_TYPE)cmbVarType.SelectedItem;
            PI_VAR_UNION varUnion = new PI_VAR_UNION();

            switch (varT)
            {
                case VAR_TYPE.BYTE:
                    varUnion._byte = Byte.Parse(tbVarValue.Text);
                    break;
                case VAR_TYPE.S_BYTE:
                    varUnion.s_byte = SByte.Parse(tbVarValue.Text);
                    break;
                case VAR_TYPE.SHORT:
                    varUnion._int16 = Int16.Parse(tbVarValue.Text);
                    break;
                case VAR_TYPE.USHORT:
                    varUnion._uint16 = UInt16.Parse(tbVarValue.Text);
                    break;
                case VAR_TYPE.INT:
                    varUnion._int32 = Int32.Parse(tbVarValue.Text);
                    break;
                case VAR_TYPE.UINT:
                    varUnion._uint32 = UInt32.Parse(tbVarValue.Text);
                    break;
                case VAR_TYPE.FLOAT:
                    varUnion._single = Single.Parse(tbVarValue.Text);
                    break;
                case VAR_TYPE.BYTE_ARR:
                    byte[] wArr = new byte[3];
                    int i = 0,data_index;
                    str = tbByteArr.Text;
                    while ((data_index = str.IndexOf(',')) != -1)
                    {
                        wArr[i] = Byte.Parse(str.Substring(0, data_index));
                        str = str.Substring(data_index + 1);
                        i++;
                        if (i == 2)
                            break;
                    }
                    wArr[i] = Byte.Parse(str.Substring(0,1));

                    varUnion._rawData = wArr;
                    //for (int j = 0; j < 3; j++)
                    //    varUnion._rawData[j] = wArr[j];
                    break;
            }
            try
            {
                ConnectData.SingleAxis.WritePIVar(index, varUnion, varT);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("Write finish success");
        }

        private void btnReadMemory_Click(object sender, EventArgs e)
        {
            string str = tbAddr.Text;
            UInt16 address;
            if (!UInt16.TryParse(str, out address))
            {
                MessageBox.Show("Address reading fail");
                return;
            }

            str = tbLength.Text;
            Byte len;
            if (!Byte.TryParse(str, out len))
            {
                MessageBox.Show("Length reading fail");
                return;
            }
            byte[] resArr = new byte[len];
            try
            {
                ConnectData.SingleAxis.EthercatReadMemoryRange(address, 8, out resArr);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            char[] ch = new char[resArr.Length];
            for (int i = 0; i < ch.Length; i++)
                ch[i] = (char) resArr[i];

            string outString = string.Empty;
            for (int i = 0; i < resArr.Length; i++)
            {
                outString += resArr[i].ToString() + "'";
            }
           
            ConnectData.ToOutput("Read memory success. Results= " + outString);
            
        }

        private void btnWriteMemory_Click(object sender, EventArgs e)
        {

            string str = tbWriteArr.Text;
            int index = -1;
            int i = 0;
            byte[] wArr = new byte[8];
            while ((index = str.IndexOf(',')) != -1)
            {
                wArr[i]=Byte.Parse(str.Substring(0, index));
                str = str.Substring(index + 1);
                i++;
                if (i == 7)
                    break;
            }
            wArr[i] = Byte.Parse(str.Substring(0));

            str = tbLength.Text;
            Byte len;
            if (!Byte.TryParse(str, out len))
            {
                MessageBox.Show("Length reading fail");
                return;
            }

            str = tbAddr.Text;
            UInt16 address;
            if (!UInt16.TryParse(str, out address))
            {
                MessageBox.Show("Address reading fail");
                return;
            }

            try
            {
                ConnectData.SingleAxis.EthercatWriteMemoryRange(address, len, wArr);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
            }
            ConnectData.ToOutput("Write memory success.");
        }

        private void btnReadLargeVar_Click(object sender, EventArgs e)
        {
            string str = tbIndex.Text;
            UInt16 index;
            if (!UInt16.TryParse(str, out index))
            {
                MessageBox.Show("Index reading fail");
                return;
            }
            PIVarDirection dir = (PIVarDirection)cmbDirection.SelectedItem;
            LARGE_VAR_TYPE varT = (LARGE_VAR_TYPE)cmbLargeVarType.SelectedItem;
            PI_LARGE_VAR_UNION varUnion = new PI_LARGE_VAR_UNION();
            try
            {
                ConnectData.SingleAxis.ReadLargePIVar(index, dir, varT, ref varUnion);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            switch (varT)
            {
                case LARGE_VAR_TYPE.DOUBLE:
                    tbVarValue.Text = varUnion._double.ToString();
                    break;
                case LARGE_VAR_TYPE.UINT64:
                    tbVarValue.Text = varUnion._uint64.ToString();
                    break;
                case LARGE_VAR_TYPE.INT64:
                    tbVarValue.Text = varUnion._int64.ToString();
                    break;
                case LARGE_VAR_TYPE.BYTE_ARR:
                    string outString = "Read Large Byte Array completed. Results= '";
                    for (int i = 0; i < varUnion._rawData.Length; i++)
                    {
                        outString += varUnion._rawData[i].ToString() + "'";
                    }
                    ConnectData.ToOutput( outString);
                    break;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            string str = tbIndex.Text;
            UInt16 index;
            if (!UInt16.TryParse(str, out index))
            {
                MessageBox.Show("Index reading fail");
                return;
            }
            LARGE_VAR_TYPE varT = (LARGE_VAR_TYPE)cmbLargeVarType.SelectedItem;
            PI_LARGE_VAR_UNION varUnion = new PI_LARGE_VAR_UNION();

            switch (varT)
            {
                case LARGE_VAR_TYPE.DOUBLE:
                    varUnion._double = Double.Parse(tbVarValue.Text);
                    break;
                case LARGE_VAR_TYPE.UINT64:
                    varUnion._uint64 = UInt64.Parse(tbVarValue.Text);
                    break;
                case LARGE_VAR_TYPE.INT64:
                    varUnion._int64 = Int64.Parse(tbVarValue.Text);
                    break;

                case LARGE_VAR_TYPE.BYTE_ARR:
                    byte[] wArr = new byte[10];
                    int i = 0,data_index;
                    str = tbByteArr.Text;
                    while ((data_index = str.IndexOf(',')) != -1)
                    {
                        wArr[i] = Byte.Parse(str.Substring(0, data_index));
                        str = str.Substring(data_index + 1);
                        i++;
                        if (i == 9)
                            break;
                    }
                    wArr[i] = Byte.Parse(str.Substring(0));

                    for (int j = 0; j < 10; j++)
                        varUnion._rawData[j] = wArr[j];
                    break;
            }
            try
            {
                ConnectData.SingleAxis.WriteLargePIVar(index, varUnion, varT);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            ConnectData.ToOutput("Write finish success");
        }
    }
}
