﻿using System;
using System.Globalization;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;
using GMAS_Lib_test;

namespace MMCLibDotNET_Test_2010
{
    public partial class SingleAxisSDOControl : UserControl, IParentPanel
    {
        public SingleAxisSDOControl()
        {
            InitializeComponent();
        }

        public ConnectivityData ConnectData { get; set; }

        private Panel panel;
        public Panel GetPanel()
        {
            return panel;
        }

        public void Init(ConnectivityData data, Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += OnDataChange;
            OnDataChange();
        }

        private void OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.SingleAxis == null)
            {   
                buttonSend.Enabled = false;
                buttonSendFl.Enabled = false;
                buttonReadSDO.Enabled = false;
                buttonWriteSDO.Enabled = false;
            }
            else
            {   
                buttonSend.Enabled = true;
                buttonSendFl.Enabled = true;
                buttonReadSDO.Enabled = true;
                buttonWriteSDO.Enabled = true;
            }
        }

        private void OnSendClicked(object sender, EventArgs e)
        {
            try
            {
                int retVal = ConnectData.SingleAxis.SendIntCmdViaSDO(textBoxCmd.Text, 1000);
                textBoxRes.Text = retVal.ToString(CultureInfo.InvariantCulture);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
            }
        }

        private void OnSendFlClicked(object sender, EventArgs e)
        {
            try
            {
                float retVal = ConnectData.SingleAxis.SendFloatCmdViaSDO(textBoxCmd.Text, 1000);
                textBoxRes.Text = retVal.ToString(CultureInfo.InvariantCulture);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
            }
        }

        private void buttonReadSDO_Click(object sender, EventArgs e)
        {
            try
            {
                UInt16 objectIndex = UInt16.Parse(textBoxIndex.Text, NumberStyles.AllowHexSpecifier);
                Byte objectSubIndex = Byte.Parse(textBoxSubIndex.Text, NumberStyles.AllowHexSpecifier);
                switch (comboBoxType.Text)
                {
                    case "Byte":
                        {
                            Byte val;
                            ConnectData.SingleAxis.UploadSDO(objectIndex, objectSubIndex, out val, 500);
                            textBoxVal.Text = val.ToString(CultureInfo.InvariantCulture);
                        }
                        break;
                    case "Int16":
                        {
                            Int16 val;
                            ConnectData.SingleAxis.UploadSDO(objectIndex, objectSubIndex, out val, 500);
                            textBoxVal.Text = val.ToString(CultureInfo.InvariantCulture);
                        }
                        break;
                    case "UInt16":
                        {
                            UInt16 val;
                            ConnectData.SingleAxis.UploadSDO(objectIndex, objectSubIndex, out val, 500);
                            textBoxVal.Text = val.ToString(CultureInfo.InvariantCulture);
                        }
                        break;
                    case "Int32":
                        {
                            Int32 val;
                            ConnectData.SingleAxis.UploadSDO(objectIndex, objectSubIndex, out val, 500);
                            textBoxVal.Text = val.ToString(CultureInfo.InvariantCulture);
                        }
                        break;
                    case "UInt32":
                        {
                            UInt32 val;
                            ConnectData.SingleAxis.UploadSDO(objectIndex, objectSubIndex, out val, 500);
                            textBoxVal.Text = val.ToString(CultureInfo.InvariantCulture);
                        }
                        break;
                    case "Float":
                        {
                            Single val;
                            ConnectData.SingleAxis.UploadSDO(objectIndex, objectSubIndex, out val, 500);
                            textBoxVal.Text = val.ToString(CultureInfo.InvariantCulture);
                        }
                        break;
                }
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
            }
        }

        private void buttonWriteSDO_Click(object sender, EventArgs e)
        {
            try
            {
                UInt16 objectIndex = UInt16.Parse(textBoxIndex.Text, NumberStyles.AllowHexSpecifier);
                Byte objectSubIndex = Byte.Parse(textBoxSubIndex.Text, NumberStyles.AllowHexSpecifier);
                switch (comboBoxType.Text)
                {
                    case "Byte":
                        {
                            Byte val = Byte.Parse(textBoxVal.Text);
                            ConnectData.SingleAxis.DownloadSDO(objectIndex, objectSubIndex, val, 500);
                        }
                        break;
                    case "Int16":
                        {
                            Int16 val = Int16.Parse(textBoxVal.Text);
                            ConnectData.SingleAxis.DownloadSDO(objectIndex, objectSubIndex, val, 500);
                        }
                        break;
                    case "UInt16":
                        {
                            UInt16 val = UInt16.Parse(textBoxVal.Text);
                            ConnectData.SingleAxis.DownloadSDO(objectIndex, objectSubIndex, val, 500);
                        }
                        break;
                    case "Int32":
                        {
                            Int32 val = Int32.Parse(textBoxVal.Text);
                            ConnectData.SingleAxis.DownloadSDO(objectIndex, objectSubIndex, val, 500);
                        }
                        break;
                    case "UInt32":
                        {
                            UInt32 val = UInt32.Parse(textBoxVal.Text);
                            ConnectData.SingleAxis.DownloadSDO(objectIndex, objectSubIndex, val, 500);
                        }
                        break;
                    case "Float":
                        {
                            Single val = Single.Parse(textBoxVal.Text);
                            ConnectData.SingleAxis.DownloadSDO(objectIndex, objectSubIndex, val, 500);
                        }
                        break;
                }
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
            }
        }
    }
}
