﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class SingleAxisSyncElmoManagmentControl : UserControl, IParentPanel
    {
        public SingleAxisSyncElmoManagmentControl()
        {
            InitializeComponent();
        }

        private System.Windows.Forms.Panel panel;
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }
        public System.Windows.Forms.Panel GetPanel()
        {
            return panel;
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
            ConnectData_OnDataChange();
        }

        private void EnableButtons(bool enable)
        {
            btnGetFloatIndexParam.Enabled = enable;
            btnGetIntegerIndexParam.Enabled = enable;
            btnGetSyncParam.Enabled = enable;
            btnGetSycFloatParam.Enabled = enable;
        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.SingleAxis == null)
                EnableButtons(false);
            else
                EnableButtons(true);
            
        }

        private void btnGetSyncParam_Click(object sender, EventArgs e)
        {
            byte[] ParamName = new byte[tbParamName.Text.Length];
            ParamName.Initialize();
            for (int i = 0; i < ParamName.Length; i++)
            {
                ParamName[i] = (byte)tbParamName.Text[i];
            }

            Int32 val = -1;
            try
            {
                ConnectData.SingleAxis.ElmoGetSyncParam(ParamName, ref val);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Elmo GetSyncInteger Param command successfully Accepted. Param " + tbParamName.Text + " = " + val.ToString();
            ConnectData.ToOutput(res);
        }

        private void btnRunTest_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < 1000000; i++)
                btnGetSyncParam_Click(this, null);
        }

        private void btnGetSycFloatParam_Click(object sender, EventArgs e)
        {
            byte[] ParamName = new byte[tbFloatParamName.Text.Length];
            ParamName.Initialize();
            for (int i = 0; i < ParamName.Length; i++)
            {
                ParamName[i] = (byte)tbFloatParamName.Text[i];
            }

            float val = -1;
            try
            {
                ConnectData.SingleAxis.ElmoGetSyncParam(ParamName, ref val);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Elmo GetSyncFloatParam command successfully Accepted. Param " + tbFloatParamName.Text + " = " + val.ToString();
            ConnectData.ToOutput(res);
        }

        private void btnGetIntegerIndexParam_Click(object sender, EventArgs e)
        {
            byte[] ParamName = new byte[tbIntegerIndexParamName.Text.Length];
            ParamName.Initialize();
            for (int i = 0; i < ParamName.Length; i++)
            {
                ParamName[i] = (byte)tbIntegerIndexParamName.Text[i];
            }
            short index;// = Int16.Parse(tbArrIndex.Text);
            if (!Int16.TryParse(tbIntegerIndex.Text, out index))
            {
                MessageBox.Show("Invalid Index");
                return;
            }

            int val = -1;
            try
            {
                ConnectData.SingleAxis.ElmoGetSyncArray( ParamName, index, ref val );
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Elmo GetSyncIntegerArray command successfully Accepted. Param " + tbIntegerIndexParamName.Text + "[" + tbIntegerIndex.Text + "] = " + val.ToString();
            ConnectData.ToOutput(res);
        }

        private void btnGetFloatIndexParam_Click(object sender, EventArgs e)
        {
            byte[] ParamName = new byte[tbFloatIndexParamName.Text.Length];
            ParamName.Initialize();
            for (int i = 0; i < ParamName.Length; i++)
            {
                ParamName[i] = (byte)tbFloatIndexParamName.Text[i];
            }
            short index;// = Int16.Parse(tbArrIndex.Text);
            if (!Int16.TryParse(tbFloatIndex.Text, out index))
            {
                MessageBox.Show("Invalid Index");
                return;
            }

            float val = -1;
            try
            {
                ConnectData.SingleAxis.ElmoGetSyncArray(ParamName, index, ref val);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Elmo GetSyncFloatArray command successfully Accepted. Param " + tbFloatIndexParamName.Text + "[" + tbFloatIndex.Text + "] = " + val.ToString();
            ConnectData.ToOutput(res);
        }

        

       
    }
}
